"""
Components identify Satellite or Satellite Capsule
==================================================

An ``IsSatellite`` component is valid if the
:py:class:`insights.combiners.satellite_version.SatelliteVersion` combiner
indicates the host is a Satellite host, and also checks the Satellite major
or major and minor versions match the specified versions when they exist.
Otherwise, it raises a :py:class:`insights.core.exceptions.SkipComponent` to prevent
dependent components from executing.

An ``IsCapsule`` component is valid if the
:py:class:`insights.combiners.satellite_version.CapsuleVersion` combiner
indicates the host is a Satellite Capsule host, and also checks the Satellite
Capsule major or major and minor versions match the specified versions when
they exist. Otherwise, it raises a :py:class:`insights.core.exceptions.SkipComponent`
to prevent dependent components from executing.
"""
from insights.combiners.satellite_version import SatelliteVersion, CapsuleVersion
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import component


@component(SatelliteVersion)
class IsSatellite(object):
    """
    This component uses ``SatelliteVersion`` combiner to determine if the host
    is a Satellite host, and also if `major_ver` is passed, it checks if the
    curent satellite major version match the argument, and if both `major_ver`
    and `minor_ver` are passed, it checks if the current satellite major and
    minor versions match both the arguments, and raises ``SkipComponent`` when
    they do not match.

    Raises:
        ParseException: When only minor_ver specified.
        SkipComponent: When Satellite major or minor versions do not match
            the arguments.
    """
    def __init__(self, sat, major_ver=None, minor_ver=None):
        if major_ver is not None:
            if sat.major != major_ver:
                raise SkipComponent("Not a Satellite {major_ver} host.".format(
                    major_ver=major_ver))
            if minor_ver is not None:
                if sat.minor != minor_ver:
                    raise SkipComponent("Not a Satellite {major_ver}.{minor_ver} host.".format(
                        major_ver=major_ver, minor_ver=minor_ver))
        else:
            if minor_ver is not None:
                raise ParseException('Can not specify the minor_ver only.')


@component(CapsuleVersion)
class IsCapsule(object):
    """
    This component uses ``CapsuleVersion`` combiner to determine if the host
    is a Satellite Capsule host, and also if `major_ver` is passed, it checks
    if the curent capsule major version match the argument, and if both `major_ver`
    and `minor_ver` are passed, it checks if both the current capsule major and
    minor versions match both the arguments, and raises ``SkipComponent`` when
    they do not match.

    Raises:
        ParseException: When only minor_ver specified.
        SkipComponent: When the Satellite Capsule major or minor versions do
            not match the arguments.
    """
    def __init__(self, cap, major_ver=None, minor_ver=None):
        if major_ver is not None:
            if cap.major != major_ver:
                raise SkipComponent("Not a Satellite Capsule {major_ver} host.".format(
                    major_ver=major_ver))
            if minor_ver is not None:
                if cap.minor != minor_ver:
                    raise SkipComponent("Not a Satellite Capsule {major_ver}.{minor_ver} host.".format(
                        major_ver=major_ver, minor_ver=minor_ver))
        else:
            if minor_ver is not None:
                raise ParseException('Can not specify the minor_ver only.')


@component(SatelliteVersion)
class IsSatellite611(IsSatellite):
    """
    This component uses ``SatelliteVersion`` combiner
    to determine the Satellite version. It checks if the Satellite version is 6.11,
    and raises ``SkipComponent`` when it isn't.

    Raises:
        SkipComponent: When the Satellite version is not 6.11.
    """
    def __init__(self, sat):
        super(IsSatellite611, self).__init__(sat, 6, 11)



"""
Module for executing a command or pipeline of commands and yielding the result
as a generator of lines.
"""
import os
import shlex
import signal
from contextlib import contextmanager
from subprocess import Popen, PIPE, STDOUT

from insights.util import which

stream_options = {
    "bufsize": -1,  # use OS defaults. Non buffered if not set.
    "universal_newlines": True,  # convert all to "\n"
    "stdout": PIPE,  # pipe to Popen.stdout instead of literally stdout
    "stderr": STDOUT  # redirect stderr to stdout for all processes
}


def reader(stream):
    for line in stream:
        yield line.rstrip("\n")


timeout_command = [which("timeout"), "-s", str(signal.SIGKILL)]


@contextmanager
def stream(command, stdin=None, env=os.environ, timeout=None):
    """
    Yields a generator of a command's output. For line oriented commands only.

    Args:
        command (str or list): a command without pipes. If it's not a list,
            ``shlex.split`` is applied.
        stdin (file like object): stream to use as the command's standard input.
        env (dict): The environment in which to execute the command. PATH should
            be defined.
        timeout (int): Amount of time in seconds to give the command to complete.
            The ``timeout`` utility must be installed to use this feature.

    Yields:
        The output stream for the command. It should typically be wrapped in a
        ``reader``.
    """
    if not isinstance(command, list):
        command = shlex.split(command)

    cmd = which(command[0])
    if cmd is None:
        path = env.get("PATH", "")
        raise Exception("Command [%s] not in PATH [%s]" % (command[0], path))

    command[0] = cmd

    if timeout:
        if not timeout_command[0]:
            raise Exception("Timeout specified but timeout command not available.")
        command = timeout_command + [str(timeout)] + command

    output = None
    try:
        output = Popen(command, env=env, stdin=stdin, **stream_options)
        yield output.stdout
    finally:
        if output:
            output.wait()


@contextmanager
def connect(*cmds, **kwargs):
    """
    Connects multiple command streams together and yields the final stream.

    Args:
        cmds (list): list of commands to pipe together. Each command will be an
            input to ``stream``.
        stdin (file like object): stream to use as the first command's
            standard input.
        env (dict): The environment in which to execute the commands. PATH
            should be defined.
        timeout (int): Amount of time in seconds to give the pipeline to complete.
            The ``timeout`` utility must be installed to use this feature.

    Yields:
        The output stream for the final command in the pipeline. It should
        typically be wrapped in a ``reader``.
    """
    stdin = kwargs.get("stdin")
    env = kwargs.get("env", os.environ)
    timeout = kwargs.get("timeout")
    end = len(cmds) - 1

    @contextmanager
    def inner(idx, inp):
        with stream(cmds[idx], stdin=inp, env=env, timeout=timeout) as s:
            if idx == end:
                yield s
            else:
                with inner(idx + 1, s) as c:
                    yield c

    with inner(0, stdin) as s:
        yield s



import time

from insights.core import dr
from insights.core.dr import run
from insights.core.context import HostContext, SosArchiveContext
from insights.core.exceptions import TimeoutException
from insights.core.plugins import datasource, make_info, rule
from insights.core.spec_factory import DatasourceProvider, RegistryPoint, SpecSet, foreach_execute


class Specs(SpecSet):
    spec_ds_timeout_1 = RegistryPoint()
    spec_ds_timeout_2 = RegistryPoint()
    spec_ds_timeout_default = RegistryPoint()
    spec_foreach_ds_timeout_1 = RegistryPoint(multi_output=True)


@datasource(timeout=1)
def foreach_ds_timeout_1(broker):
    time.sleep(2)
    return ['test1', 'test2', 'test3']


@datasource(timeout=1)
def ds_timeout_1(broker):
    time.sleep(2)
    return DatasourceProvider('foo', "test_ds_timeout_1")


@datasource(timeout=3)
def ds_timeout_2(broker):
    time.sleep(1)
    return DatasourceProvider('foo', "test_ds_timeout_2")


@datasource()
def ds_timeout_default(broker):
    time.sleep(1)
    return DatasourceProvider('foo', "test_ds_timeout_def")


class TestSpecs(Specs):
    spec_ds_timeout_1 = ds_timeout_1
    spec_ds_timeout_2 = ds_timeout_2
    spec_ds_timeout_default = ds_timeout_default
    spec_foreach_ds_timeout_1 = foreach_execute(foreach_ds_timeout_1, "/usr/bin/echo %s")


@rule(Specs.spec_ds_timeout_2, Specs.spec_ds_timeout_default)
def timeout_datasource_no_timeout(ds_to_2, ds_to_def):
    return make_info('INFO_1')


@rule(Specs.spec_ds_timeout_1)
def timeout_datasource_hit(ds_to_1):
    return make_info('INFO_2')


@rule(Specs.spec_foreach_ds_timeout_1)
def timeout_foreach_datasource_hit(foreach_ds_to_1):
    return make_info('INFO_2')


def test_timeout_datasource_no_hit():
    broker = dr.Broker()
    broker[HostContext] = HostContext
    broker = run(timeout_datasource_no_timeout, broker=broker)
    assert timeout_datasource_no_timeout in broker
    assert Specs.spec_ds_timeout_2 not in broker.exceptions


def test_timeout_datasource_hit_def():
    broker = dr.Broker()
    broker[HostContext] = HostContext
    broker = run(timeout_datasource_hit, broker=broker)
    assert timeout_datasource_hit in broker
    assert Specs.spec_ds_timeout_1 in broker.exceptions
    exs = broker.exceptions[Specs.spec_ds_timeout_1]
    assert [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) == "Datasource spec insights.tests.datasources.test_datasource_timeout.TestSpecs.spec_ds_timeout_1 timed out after 1 seconds!"]


def test_timeout_foreach_datasource_hit_def():
    broker = dr.Broker()
    broker[HostContext] = HostContext
    broker = run(timeout_foreach_datasource_hit, broker=broker)
    assert timeout_foreach_datasource_hit in broker
    assert Specs.spec_foreach_ds_timeout_1 in broker.exceptions
    exs = broker.exceptions[Specs.spec_foreach_ds_timeout_1]
    assert [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) == "Datasource spec insights.tests.datasources.test_datasource_timeout.foreach_ds_timeout_1 timed out after 1 seconds!"]


def test_not_hostcontext_timeout_datasource_hit_def():
    broker = dr.Broker()
    broker[SosArchiveContext] = SosArchiveContext
    broker = run(timeout_datasource_hit, broker=broker)
    assert timeout_datasource_hit in broker
    assert Specs.spec_ds_timeout_1 not in broker.exceptions
    exs = broker.exceptions[Specs.spec_ds_timeout_1]
    assert not [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) != "Datasource spec insights.tests.datasources.test_datasource_timeout.TestSpecs.spec_ds_timeout_1 timed out after 1 seconds!"]


def test_not_hostcontext_timeout_foreach_datasource_hit_def():
    broker = dr.Broker()
    broker[SosArchiveContext] = SosArchiveContext
    broker = run(timeout_foreach_datasource_hit, broker=broker)
    assert timeout_foreach_datasource_hit in broker
    assert Specs.spec_foreach_ds_timeout_1 not in broker.exceptions
    exs = broker.exceptions[Specs.spec_foreach_ds_timeout_1]
    assert not [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) != "Datasource spec insights.tests.datasources.test_datasource_timeout.foreach_ds_timeout_1 timed out after 1 seconds!"]



import doctest
from insights.parsers import neutron_metadata_agent_log
from insights.parsers.neutron_metadata_agent_log import NeutronMetadataAgentLog
from insights.tests import context_wrap

from datetime import datetime


METADATA_AGENT_LOG = """
2018-06-08 17:29:55.894 11770 WARNING neutron.agent.metadata.agent [-] Server does not support metadata RPC, fallback to using neutron client
2018-06-08 17:29:55.907 11770 ERROR neutron.agent.metadata.agent [-] Unexpected error.
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent Traceback (most recent call last):
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent   File "/usr/lib/python2.7/site-packages/neutron/agent/metadata/agent.py", line 109, in __call__
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent     self._authenticate_keystone()
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent   File "/usr/lib/python2.7/site-packages/neutronclient/client.py", line 218, in _authenticate_keystone
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent     raise exceptions.Unauthorized(message=resp_body)
2018-06-08 17:29:56.126 11770 TRACE neutron.agent.metadata.agent Unauthorized: {"error": {"message": "The resource could not be found.", "code": 404, "title": "Not Found"}}
""".strip()


def test_neutron_metadata_agent_log():
    nmda_log = NeutronMetadataAgentLog(context_wrap(METADATA_AGENT_LOG))
    assert len(nmda_log.get("Server does not support metadata RPC, fallback to using neutron client")) == 1
    assert len(list(nmda_log.get_after(datetime(2018, 6, 8, 17, 29, 56)))) == 6
    assert len(nmda_log.get('TRACE')) == 6
    assert 'The resource could not be found' in nmda_log


def test_doc():
    env = {'metadata_agent_log': NeutronMetadataAgentLog(context_wrap(METADATA_AGENT_LOG, path='/var/log/neutron/metadata-agent.log'))}
    failed, total = doctest.testmod(neutron_metadata_agent_log, globs=env)
    assert failed == 0



"""
AvcCacheThreshold - File ``/sys/fs/selinux/avc/cache_threshold``
================================================================

This parser reads the content of ``/sys/fs/selinux/avc/cache_threshold``.
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.avc_cache_threshold)
class AvcCacheThreshold(CommandParser):
    """
    Class ``AvcCacheThreshold`` parses the content of the ``/sys/fs/selinux/avc/cache_threshold``.

    Attributes:
        cache_threshold (int): It is used to show the value of cache threshold.

    A typical sample of the content of this file looks like::

        512

    Examples:
        >>> type(avc_cache_threshold)
        <class 'insights.parsers.avc_cache_threshold.AvcCacheThreshold'>
        >>> avc_cache_threshold.cache_threshold
        512
    """

    def parse_content(self, content):
        if len(content) != 1:
            raise ParseException("Error: ", content[0] if content else 'empty file')
        self.cache_threshold = int(content[0].strip())



"""
SAP Trace Files  - Files ``/usr/sap/SID/SNAME/work/dev_*``
==========================================================

Parsers included in this module are:

SapDevDisp - Files ``/usr/sap/SID/SNAME/work/dev_disp``
-------------------------------------------------------

SapDevRd - Files ``/usr/sap/SID/SNAME/work/dev_rd``
---------------------------------------------------
"""
from insights.core import LogFileOutput
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


class SapDevTraceFile(LogFileOutput):
    """
    The Base class for parsing the SAP trace files.
    """

    def get_after(self, *args, **kwargs):
        """
        .. warning::
            The ``get_after`` function is not supported by this Parser because
            of the structure of the SAP trace files are totally different with
            the log files expected by the base class``LogFileOutput``.

        Raises:
            ParseException: Always raises ParseException.
        """
        raise ParseException("get_after() is not supported by this Parser.")

    @property
    def sid(self):
        """
        The SID of this trace file.
        """
        return self.file_path.lstrip('/').split('/')[2]

    @property
    def instance(self):
        """
        The instance name of this trace file.
        """
        return self.file_path.lstrip('/').split('/')[3]


@parser(Specs.sap_dev_disp)
class SapDevDisp(SapDevTraceFile):
    """
    This class reads the SAP trace files ``/usr/sap/SID/SNAME/work/dev_disp``

    Sample content of the file::

        ---------------------------------------------------
        trc file: "dev_disp", trc level: 1, release: "745"
        ---------------------------------------------------

        Sun Aug 18 17:17:45 2019
        TRACE FILE TRUNCATED ( pid =    16955 )

        sysno      00
        sid        RH1
        systemid   390 (AMD/Intel x86_64 with Linux)
        relno      7450
        patchlevel 0
        patchno    100
        intno      20151301
        make       multithreaded, Unicode, 64 bit, optimized
        profile    /usr/sap/RH1/SYS/profile/RH1_D00_vm37-39
        pid        16955

        Sun Aug 18 17:17:45 2019
        kernel runs with dp version 3000(ext=117000) (@(#) DPLIB-INT-VERSION-0+3000-UC)
        length of sys_adm_ext is 500 bytes

        Scheduler info
        --------------
        WP info
          #dia     = 10
          #btc     = 3
          #standby = 0
          #max     = 21
        General Scheduler info
          preemptionInfo.isActive           = true
          preemptionInfo.timeslice          = 500
          preemptionInfo.checkLoad          = true
        Prio Class High
          maxRuntime[RQ_Q_PRIO_HIGH]     = 600 sec
          maxRuntimeHalf[RQ_Q_PRIO_HIGH] = 300 sec
        Running requests[RQ_Q_PRIO_NORMAL] = 0
        Running requests[RQ_Q_PRIO_LOW] = 0

        *** WARNING => DpHdlDeadWp: wp_adm slot for W7 has no pid [dpxxwp.c     1353]
        DpSkipSnapshot: last snapshot created at Sun Aug 18 17:15:25 2019, skip new snapshot
        *** WARNING => DpHdlDeadWp: wp_adm slot for W8 has no pid [dpxxwp.c     1353]
        DpSkipSnapshot: last snapshot created at Sun Aug 18 17:15:25 2019, skip new snapshot
        *** WARNING => DpHdlDeadWp: wp_adm slot for W9 has no pid [dpxxwp.c     1353]

        Sun Aug 18 17:17:45 2019
        DpSkipSnapshot: last snapshot created at Sun Aug 18 17:17:45 2019, skip new snapshot
        DpCheckSapcontrolProcess: sapcontrol with pid 1479 terminated
        *** WARNING => DpRequestProcessingCheck: potential request processing problem detected (14. check) [dpxxwp.c     4633]

    Examples:
        >>> type(dev_disp)
        <class 'insights.parsers.sap_dev_trace_files.SapDevDisp'>
        >>> dev_disp.file_path == '/usr/sap/RH1/D00/work/dev_disp'
        True
        >>> dev_disp.sid == 'RH1'
        True
        >>> dev_disp.instance == 'D00'
        True
        >>> len(dev_disp.get("WARNING"))
        4
    """
    pass


@parser(Specs.sap_dev_rd)
class SapDevRd(SapDevTraceFile):
    """
    This class reads the SAP trace files ``/usr/sap/SID/SNAME/work/dev_rd``

    Sample content of the file::

        ---------------------------------------------------
        trc file: "dev_rd", trc level: 1, release: "745"
        ---------------------------------------------------
        systemid   390 (AMD/Intel x86_64 with Linux)
        relno      7450
        patchlevel 0
        patchno    100
        intno      20151301
        make       multithreaded, Unicode, 64 bit, optimized
        pid        16982

        Thu Jul 18 02:59:37 2019
        gateway (version=745.2015.12.21 (with SSL support))
        Bind service  (socket) to port
        GwPrintMyHostAddr: my host addresses are :
        *
        * SWITCH TRC-LEVEL from 1 to 1
        *
        ***LOG S00=> GwInitReader, gateway started ( 16982) [gwxxrd.c     1820]
        systemid   390 (AMD/Intel x86_64 with Linux)
        relno      7450
        patchlevel 0
        patchno    100
        intno      20151301
        make       multithreaded, Unicode, 64 bit, optimized
        pid        16982

        Thu Jul 18 02:59:37 2019
        gateway (version=745.2015.12.21 (with SSL support))
        gw/reg_no_conn_info = 1
        * SWITCH TRC-RESOLUTION from 1 to 1
        gw/sim_mode : set to 0
        gw/logging : ACTION=Ss LOGFILE=gw_log-%y-%m-%d SWITCHTF=day MAXSIZEKB=100
        NI buffering enabled
        CCMS: initialize CCMS Monitoring for ABAP instance with J2EE addin.

        Thu Jul 18 02:59:38 2019
        CCMS: Initialized monitoring segment of size 60000000.
        CCMS: Initialized CCMS Headers in the shared monitoring segment.
        CCMS: Checking Downtime Configuration of Monitoring Segment.

        Thu Jul 18 02:59:39 2019
        CCMS: AlMsUpload called by wp 1002.
        CCMS: AlMsUpload successful for /usr/sap/RH1/D00/log/ALMTTREE (542 MTEs).

        Thu Jul 18 02:59:40 2019
        GwIInitSecInfo: secinfo version = 2
        GwIRegInitRegInfo: reg_info file /usr/sap/RH1/D00/data/reginfo not found

        ********** SERVER SNAPSHOT 2727 - begin **********

        -----------------------
        --- SYS TABLE DUMP ----
        -----------------------
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+
        | Indx | State | Hostname             | Addr            | Port / TP  | Type  | Last-Request             |
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+
        |    4 | CONNE | vm37-39              | 127.0.0.1       | IGS.RH1    | REGIS | Mon Aug 19 05:15:15 2019 |
        |    3 | CONNE | vm37-39              | 127.0.0.1       | IGS.RH1    | REGIS | Mon Aug 19 05:15:15 2019 |
        |    2 | CONNE | vm37-39              | 127.0.0.1       | IGS.RH1    | REGIS | Mon Aug 19 05:15:15 2019 |
        |    1 | CONNE | vm37-39              | 127.0.0.1       | IGS.RH1    | REGIS | Mon Aug 19 05:15:15 2019 |
        |    0 | CONNE | vm37-39.gsslab.pek2. | 10.72.37.39     | sapgw00    | LOCAL | Wed Aug  7 22:58:50 2019 |
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+
        -----------------------
        --- GWTBL TABLE DUMP --
        -----------------------
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+
        | Indx | State | Hostname             | Addr            | Port       | Type  | Last-Request             |
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+
        |    0 | CONNE | vm37-39.gsslab.pek2. | 10.72.37.39     | sapgw00    | LOCAL | Wed Aug  7 22:58:50 2019 |
        +------+-------+----------------------+-----------------+------------+-------+--------------------------+

        ********** SERVER SNAPSHOT 2727 - end ************

    Examples:
        >>> type(dev_rd)
        <class 'insights.parsers.sap_dev_trace_files.SapDevRd'>
        >>> dev_rd.file_path == '/usr/sap/RH2/D03/work/dev_rd'
        True
        >>> dev_rd.sid == 'RH2'
        True
        >>> dev_rd.instance == 'D03'
        True
        >>> len(dev_rd.get("CCMS:"))
        6
    """
    pass



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import podman_inspect
from insights.tests import context_wrap


PODMAN_CONTAINER_INSPECT = """
[
    {
        "ID": "66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda",
        "Created": "2019-08-21T10:38:34.753548542Z",
        "Path": "dumb-init",
        "Args": [
            "--single-child",
            "--",
            "kolla_start"
        ],
        "State": {
            "OciVersion": "1.0.1-dev",
            "Status": "running",
            "Running": true,
            "Paused": false,
            "Restarting": false,
            "OOMKilled": false,
            "Dead": false,
            "Pid": 6606,
            "ExitCode": 0,
            "Error": "",
            "StartedAt": "2019-09-06T19:16:08.066138727Z",
            "FinishedAt": "0001-01-01T00:00:00Z"
        },
        "Image": "a6b8f27df9feb9d820527d413f24ec9b1fcfb12049dd91af5fc188636bebe504",
        "ImageName": "192.168.24.1:8787/rhosp15/openstack-gnocchi-metricd:20190819.1",
        "Rootfs": "",
        "ResolvConfPath": "/var/run/containers/storage/overlay-containers/66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda/userdata/resolv.conf",
        "HostnamePath": "/var/run/containers/storage/overlay-containers/66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda/userdata/hostname",
        "HostsPath": "/var/run/containers/storage/overlay-containers/66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda/userdata/hosts",
        "StaticDir": "/var/lib/containers/storage/overlay-containers/66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda/userdata",
        "LogPath": "/var/log/containers/stdouts/gnocchi_metricd.log",
        "Name": "gnocchi_metricd",
        "RestartCount": 0,
        "Driver": "overlay",
        "MountLabel": "system_u:object_r:container_file_t:s0:c514,c813",
        "ProcessLabel": "system_u:system_r:container_t:s0:c514,c813",
        "AppArmorProfile": "",
        "EffectiveCaps": null,
        "BoundingCaps": [
            "CAP_CHOWN",
            "CAP_DAC_OVERRIDE",
            "CAP_FSETID",
            "CAP_FOWNER",
            "CAP_MKNOD",
            "CAP_NET_RAW",
            "CAP_SETGID",
            "CAP_SETUID",
            "CAP_SETFCAP",
            "CAP_SETPCAP",
            "CAP_NET_BIND_SERVICE",
            "CAP_SYS_CHROOT",
            "CAP_KILL",
            "CAP_AUDIT_WRITE"
        ],
        "ExecIDs": [],
        "GraphDriver": {
            "Name": "overlay",
            "Data": {
                "LowerDir": "/var/lib/containers/storage/overlay/8cf325ff3583d7b6e10f170f85605e14797460f4ee0fa8d4eef2176c27627a26/diff:/var/lib/containers/storage/overlay/3fcda38b3b3199f8d0a1aa61a20d9561ba4ca4805e09ae18d6b50f2854cd5091/diff:/var/lib/containers/storage/overlay/1344d596d37069ebcdd447b67559396b6d046f2f98a63093f229621c544da013/diff:/var/lib/containers/storage/overlay/4faa8d5827f59db011a639ea73621234812291ff51d875f58e1e4197c6239429/diff:/var/lib/containers/storage/overlay/671441d9601355a777b2ce9afceef5a7d0d4890d11ef4f5744e534394ef7c447/diff:/var/lib/containers/storage/overlay/c7269138daa6b23e16efdcbf7ee323ba42fba93eb2406192ac22631bdd0cb4e3/diff",
                "MergedDir": "/var/lib/containers/storage/overlay/87b994364ae69db1d3d8ff1e19a5882f230514bce4a3362ee25bfe618f9fa5ee/merged",
                "UpperDir": "/var/lib/containers/storage/overlay/87b994364ae69db1d3d8ff1e19a5882f230514bce4a3362ee25bfe618f9fa5ee/diff",
                "WorkDir": "/var/lib/containers/storage/overlay/87b994364ae69db1d3d8ff1e19a5882f230514bce4a3362ee25bfe618f9fa5ee/work"
            }
        },
        "Mounts": [
            {
                "destination": "/sys",
                "type": "sysfs",
                "source": "sysfs",
                "options": [
                    "nosuid",
                    "noexec",
                    "nodev",
                    "ro"
                ]
            },
            {
                "destination": "/dev",
                "type": "tmpfs",
                "source": "tmpfs",
                "options": [
                    "nosuid",
                    "strictatime",
                    "mode=755",
                    "size=65536k"
                ]
            },
            {
                "destination": "/etc/pki/ca-trust/source/anchors",
                "type": "bind",
                "source": "/etc/pki/ca-trust/source/anchors",
                "options": [
                    "ro",
                    "rbind",
                    "rprivate"
                ]
            },
            {
                "destination": "/var/lib/kolla/config_files/src-ceph",
                "type": "bind",
                "source": "/etc/ceph",
                "options": [
                    "ro",
                    "rbind",
                    "rprivate"
                ]
            }
        ],
        "Dependencies": [],
        "NetworkSettings": {
            "Bridge": "",
            "SandboxID": "",
            "HairpinMode": false,
            "LinkLocalIPv6Address": "",
            "LinkLocalIPv6PrefixLen": 0,
            "Ports": [],
            "SandboxKey": "",
            "SecondaryIPAddresses": null,
            "SecondaryIPv6Addresses": null,
            "EndpointID": "",
            "Gateway": "",
            "GlobalIPv6Address": "",
            "GlobalIPv6PrefixLen": 0,
            "IPAddress": "",
            "IPPrefixLen": 0,
            "IPv6Gateway": "",
            "MacAddress": ""
        },
        "ExitCommand": [
            "/usr/bin/podman",
            "--root",
            "/var/lib/containers/storage",
            "--runroot",
            "/var/run/containers/storage",
            "--log-level",
            "error",
            "--cgroup-manager",
            "systemd",
            "--tmpdir",
            "/var/run/libpod",
            "--storage-driver",
            "overlay",
            "container",
            "cleanup",
            "66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda"
        ],
        "Namespace": "",
        "IsInfra": false,
        "HostConfig": {
            "ContainerIDFile": "",
            "LogConfig": null,
            "NetworkMode": "host",
            "PortBindings": null,
            "AutoRemove": false,
            "CapAdd": [],
            "CapDrop": [],
            "DNS": [],
            "DNSOptions": [],
            "DNSSearch": [],
            "ExtraHosts": null,
            "GroupAdd": null,
            "IpcMode": "",
            "Cgroup": "host",
            "OomScoreAdj": 0,
            "PidMode": "",
            "Privileged": false,
            "PublishAllPorts": false,
            "ReadonlyRootfs": false,
            "SecurityOpt": [],
            "UTSMode": "",
            "UsernsMode": "",
            "ShmSize": 65536000,
            "Runtime": "runc",
            "ConsoleSize": null,
            "CpuShares": null,
            "Memory": 0,
            "NanoCpus": 0,
            "CgroupParent": "",
            "BlkioWeight": null,
            "BlkioWeightDevice": null,
            "BlkioDeviceReadBps": null,
            "BlkioDeviceWriteBps": null,
            "BlkioDeviceReadIOps": null,
            "BlkioDeviceWriteIOps": null,
            "CpuPeriod": null,
            "CpuQuota": null,
            "CpuRealtimePeriod": null,
            "CpuRealtimeRuntime": null,
            "CpuSetCpus": "",
            "CpuSetMems": "",
            "Devices": null,
            "DiskQuota": 0,
            "KernelMemory": null,
            "MemoryReservation": null,
            "MemorySwap": null,
            "MemorySwappiness": null,
            "OomKillDisable": false,
            "PidsLimit": null,
            "Ulimits": [],
            "CpuCount": 0,
            "CpuPercent": 0,
            "IOMaximumIOps": 0,
            "IOMaximumBandwidth": 0,
            "Tmpfs": []
        },
        "Config": {
            "Hostname": "controller-0",
            "Domainname": "",
            "User": {
                "uid": 0,
                "gid": 0
            },
            "AttachStdin": false,
            "AttachStdout": false,
            "AttachStderr": false,
            "Tty": false,
            "OpenStdin": false,
            "StdinOnce": false,
            "Env": [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "TERM=xterm",
                "HOSTNAME=controller-0",
                "container=oci",
                "KOLLA_CONFIG_STRATEGY=COPY_ALWAYS",
                "TRIPLEO_CONFIG_HASH=3faca5d7029273bb994631cb4a075e0f",
                "KOLLA_INSTALL_TYPE=binary",
                "KOLLA_INSTALL_METATYPE=rhos",
                "KOLLA_DISTRO_PYTHON_VERSION=3.6",
                "KOLLA_BASE_DISTRO=rhel",
                "PS1=$(tput bold)($(printenv KOLLA_SERVICE_NAME))$(tput sgr0)[$(id -un)@$(hostname -s) $(pwd)]$ "
            ],
            "Cmd": [
                "dumb-init",
                "--single-child",
                "--",
                "kolla_start"
            ],
            "Image": "192.168.24.1:8787/rhosp15/openstack-gnocchi-metricd:20190819.1",
            "Volumes": null,
            "WorkingDir": "/",
            "Entrypoint": "dumb-init --single-child --",
            "Labels": {
                "architecture": "x86_64",
                "authoritative-source-url": "registry.access.redhat.com",
                "batch": "20190819.1",
                "build-date": "2019-08-19T20:42:03.096048",
                "com.redhat.build-host": "cpt-1004.osbs.prod.upshift.rdu2.redhat.com",
                "com.redhat.component": "openstack-gnocchi-metricd-container",
                "com.redhat.license_terms": "https://www.redhat.com/en/about/red-hat-end-user-license-agreements",
                "config_id": "tripleo_step5",
                "container_name": "gnocchi_metricd",
                "description": "Red Hat OpenStack Platform 15.0 gnocchi-metricd",
                "distribution-scope": "public",
                "io.k8s.description": "Red Hat OpenStack Platform 15.0 gnocchi-metricd",
                "io.k8s.display-name": "Red Hat OpenStack Platform 15.0 gnocchi-metricd",
                "io.openshift.expose-services": "",
                "io.openshift.tags": "rhosp osp openstack osp-15.0",
                "maintainer": "Red Hat, Inc.",
                "managed_by": "paunch",
                "name": "rhosp15/openstack-gnocchi-metricd",
                "release": "58",
                "summary": "Red Hat OpenStack Platform 15.0 gnocchi-metricd",
                "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhosp15/openstack-gnocchi-metricd/images/15.0-58",
                "vcs-ref": "18e6ecd9e04f6590526657b85423347b7543391a",
                "vcs-type": "git",
                "vendor": "Red Hat, Inc.",
                "version": "15.0"
            },
            "Annotations": {
                "io.kubernetes.cri-o.ContainerType": "sandbox",
                "io.kubernetes.cri-o.TTY": "false"
            },
            "StopSignal": 15
        }
    }
]
""".splitlines()

PODMAN_IMAGE_INSPECT = """
[
    {
        "Id": "013125b8a088f45be8f85f88b5504f05c02463b10a6eea2b66809a262bb911ca",
        "Digest": "sha256:f9662cdd45e3db182372a4fa6bfff10e1c601cc785bac09ccae3b18f0bc429df",
        "RepoTags": [
            "192.168.24.1:8787/rhosp15/openstack-rabbitmq:20190819.1",
            "192.168.24.1:8787/rhosp15/openstack-rabbitmq:pcmklatest"
        ],
        "RepoDigests": [
            "192.168.24.1:8787/rhosp15/openstack-rabbitmq@sha256:f9662cdd45e3db182372a4fa6bfff10e1c601cc785bac09ccae3b18f0bc429df",
            "192.168.24.1:8787/rhosp15/openstack-rabbitmq@sha256:f9662cdd45e3db182372a4fa6bfff10e1c601cc785bac09ccae3b18f0bc429df"
        ],
        "Parent": "",
        "Comment": "",
        "Created": "2019-08-19T19:39:31.939714Z",
        "Config": {
            "User": "rabbitmq",
            "Env": [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "container=oci",
                "KOLLA_BASE_DISTRO=rhel",
                "KOLLA_INSTALL_TYPE=binary",
                "KOLLA_INSTALL_METATYPE=rhos",
                "KOLLA_DISTRO_PYTHON_VERSION=3.6",
                "PS1=$(tput bold)($(printenv KOLLA_SERVICE_NAME))$(tput sgr0)[$(id -un)@$(hostname -s) $(pwd)]$ "
            ],
            "Entrypoint": [
                "dumb-init",
                "--single-child",
                "--"
            ],
            "Cmd": [
                "kolla_start"
            ],
            "Labels": {
                "architecture": "x86_64",
                "authoritative-source-url": "registry.access.redhat.com",
                "batch": "20190819.1",
                "build-date": "2019-08-19T19:38:18.798307",
                "com.redhat.build-host": "cpt-1003.osbs.prod.upshift.rdu2.redhat.com",
                "com.redhat.component": "openstack-rabbitmq-container",
                "com.redhat.license_terms": "https://www.redhat.com/en/about/red-hat-end-user-license-agreements",
                "description": "Red Hat OpenStack Platform 15.0 rabbitmq",
                "distribution-scope": "public",
                "io.k8s.description": "Red Hat OpenStack Platform 15.0 rabbitmq",
                "io.k8s.display-name": "Red Hat OpenStack Platform 15.0 rabbitmq",
                "io.openshift.expose-services": "",
                "io.openshift.tags": "rhosp osp openstack osp-15.0",
                "maintainer": "Red Hat, Inc.",
                "name": "rhosp15/openstack-rabbitmq",
                "release": "64",
                "summary": "Red Hat OpenStack Platform 15.0 rabbitmq",
                "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhosp15/openstack-rabbitmq/images/15.0-64",
                "vcs-ref": "292efe508dcdf588e92503273c5abcc89af574d6",
                "vcs-type": "git",
                "vendor": "Red Hat, Inc.",
                "version": "15.0"
            },
            "StopSignal": "SIGTERM"
        },
        "Version": "1.13.1",
        "Author": "",
        "Architecture": "amd64",
        "Os": "linux",
        "Size": 542316943,
        "VirtualSize": 542316943,
        "GraphDriver": {
            "Name": "overlay",
            "Data": {
                "LowerDir": "/var/lib/containers/storage/overlay/4faa8d5827f59db011a639ea73621234812291ff51d875f58e1e4197c6239429/diff:/var/lib/containers/storage/overlay/671441d9601355a777b2ce9afceef5a7d0d4890d11ef4f5744e534394ef7c447/diff:/var/lib/containers/storage/overlay/c7269138daa6b23e16efdcbf7ee323ba42fba93eb2406192ac22631bdd0cb4e3/diff",
                "MergedDir": "/var/lib/containers/storage/overlay/b25ec647038ff1d35285422ddbedb5b5c7d36d64e67fcbbd2a2f205dc2aa1eb5/merged",
                "UpperDir": "/var/lib/containers/storage/overlay/b25ec647038ff1d35285422ddbedb5b5c7d36d64e67fcbbd2a2f205dc2aa1eb5/diff",
                "WorkDir": "/var/lib/containers/storage/overlay/b25ec647038ff1d35285422ddbedb5b5c7d36d64e67fcbbd2a2f205dc2aa1eb5/work"
            }
        },
        "RootFS": {
            "Type": "layers",
            "Layers": [
                "sha256:c7269138daa6b23e16efdcbf7ee323ba42fba93eb2406192ac22631bdd0cb4e3",
                "sha256:786011f2f6269cc2512d58fd7d6c8feac1330754b12b4ffacfcaa8bd685ed898",
                "sha256:d74075ef7bbc7f840dec3cafc7bf8f82e900ee2f8b4a4d328448965bd8e398ce",
                "sha256:272314807b476c2c183edd6427bd450cea885976446afcdbd6b52ad47943a60f"
            ]
        },
        "Labels": {
            "architecture": "x86_64",
            "authoritative-source-url": "registry.access.redhat.com",
            "batch": "20190819.1",
            "build-date": "2019-08-19T19:38:18.798307",
            "com.redhat.build-host": "cpt-1003.osbs.prod.upshift.rdu2.redhat.com",
            "com.redhat.component": "openstack-rabbitmq-container",
            "com.redhat.license_terms": "https://www.redhat.com/en/about/red-hat-end-user-license-agreements",
            "description": "Red Hat OpenStack Platform 15.0 rabbitmq",
            "distribution-scope": "public",
            "io.k8s.description": "Red Hat OpenStack Platform 15.0 rabbitmq",
            "io.k8s.display-name": "Red Hat OpenStack Platform 15.0 rabbitmq",
            "io.openshift.expose-services": "",
            "io.openshift.tags": "rhosp osp openstack osp-15.0",
            "maintainer": "Red Hat, Inc.",
            "name": "rhosp15/openstack-rabbitmq",
            "release": "64",
            "summary": "Red Hat OpenStack Platform 15.0 rabbitmq",
            "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhosp15/openstack-rabbitmq/images/15.0-64",
            "vcs-ref": "292efe508dcdf588e92503273c5abcc89af574d6",
            "vcs-type": "git",
            "vendor": "Red Hat, Inc.",
            "version": "15.0"
        },
        "Annotations": {},
        "ManifestType": "application/vnd.docker.distribution.manifest.v2+json",
        "User": "rabbitmq",
        "History": [
            {
                "created": "2019-07-15T05:10:57.589513378Z",
                "comment": "Imported from -"
            },
            {
                "created": "2019-07-15T05:11:04.220661Z"
            },
            {
                "created": "2019-08-19T19:24:22.99993Z"
            },
            {
                "created": "2019-08-19T19:39:31.939714Z"
            }
        ]
    }
]
""".splitlines()

PODMAN_CONTAINER_INSPECT_TRUNCATED = """
[
    {
        "ID": "66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda",
        "Created": "2019-08-21T10:38:34.753548542Z",
        "Path": "dumb-init",
"""


def test_podman_object_container_inspect():
    result = podman_inspect.PodmanInspect(context_wrap(PODMAN_CONTAINER_INSPECT))
    assert result.get('ID') == "66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda"
    assert result.get('NetworkSettings').get('HairpinMode') is False
    assert result.get('Config').get('Env') == [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "TERM=xterm",
                "HOSTNAME=controller-0",
                "container=oci",
                "KOLLA_CONFIG_STRATEGY=COPY_ALWAYS",
                "TRIPLEO_CONFIG_HASH=3faca5d7029273bb994631cb4a075e0f",
                "KOLLA_INSTALL_TYPE=binary",
                "KOLLA_INSTALL_METATYPE=rhos",
                "KOLLA_DISTRO_PYTHON_VERSION=3.6",
                "KOLLA_BASE_DISTRO=rhel",
                "PS1=$(tput bold)($(printenv KOLLA_SERVICE_NAME))$(tput sgr0)[$(id -un)@$(hostname -s) $(pwd)]$ "
            ]
    assert result.get('GraphDriver').get('Name') == 'overlay'


def test_podman_object_image_inspect():
    result = podman_inspect.PodmanInspect(context_wrap(PODMAN_IMAGE_INSPECT))
    assert result.get('Id') == "013125b8a088f45be8f85f88b5504f05c02463b10a6eea2b66809a262bb911ca"
    assert result.get('Size') == 542316943
    assert result.get('Digest') == "sha256:f9662cdd45e3db182372a4fa6bfff10e1c601cc785bac09ccae3b18f0bc429df"


def test_podman_container_inspect_truncated_input():
    with pytest.raises(SkipComponent):
        podman_inspect.PodmanInspectContainer(context_wrap(PODMAN_CONTAINER_INSPECT_TRUNCATED))


def test_doc_test():
    dic = podman_inspect.PodmanInspectContainer(context_wrap(PODMAN_CONTAINER_INSPECT))
    dii = podman_inspect.PodmanInspectImage(context_wrap(PODMAN_IMAGE_INSPECT))
    env = {
            'container': dic,
            'image': dii,
    }
    failed, total = doctest.testmod(podman_inspect, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import etc_machine_id
from insights.parsers.etc_machine_id import EtcMachineId
from insights.tests import context_wrap

ETC_MACHINE_ID = """
4f3fbfda59ff4aea969255a73342d439
""".strip()

ETC_MACHINE_ID_ERROR = """
4f3fbfda59ff4aea969255a73342d439
4f3fbfda59ff4aea969255a7334
""".strip()


def test_etcd_conf_empty():
    with pytest.raises(SkipComponent):
        EtcMachineId(context_wrap(''))


def test_etcd_conf_error():
    with pytest.raises(SkipComponent):
        EtcMachineId(context_wrap(ETC_MACHINE_ID_ERROR))


def test_etcd_conf():
    machine_id = EtcMachineId(context_wrap(ETC_MACHINE_ID))
    assert machine_id.machine_id == '4f3fbfda59ff4aea969255a73342d439'


def test_etcd_conf_documentation():
    env = {
        'machine_id': EtcMachineId(context_wrap(ETC_MACHINE_ID)),
    }
    failed, total = doctest.testmod(etc_machine_id, globs=env)
    assert failed == 0



"""
LsVarSpoolPostfixMaildrop - command ``ls -ln /var/spool/postfix/maildrop``
==========================================================================

The ``ls -ln /var/spool/postfix/maildrop`` command provides information for the listing of the ``/var/spool/postfix/maildrop`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 20
    -rwxr--r--. 1 0 90 258 Jul 11 15:54 55D6821C286
    -rwxr--r--. 1 0 90 282 Jul 11 15:54 5852121C284
    -rwxr--r--. 1 0 90 258 Jul 11 15:54 9FFEC21C287
    -rwxr--r--. 1 0 90 258 Jul 11 15:54 E9A4521C285
    -rwxr--r--. 1 0 90 258 Jul 11 15:54 EA60F21C288

Examples:

    >>> "55D6821C286" in ls_var_spool_postfix_maildrop
    False
    >>> "/var/spool/postfix/maildrop" in ls_var_spool_postfix_maildrop
    True
    >>> ls_var_spool_postfix_maildrop.dir_entry('/var/spool/postfix/maildrop', '55D6821C286')['type']
    '-'
"""


from insights.specs import Specs
from insights.util import deprecated

from .. import CommandParser, parser
from .. import FileListing


@parser(Specs.ls_var_spool_postfix_maildrop)
class LsVarSpoolPostfixMaildrop(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlan` instead.

    Parses output of ``ls -ln /var/spool/postfix/maildrop`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarSpoolPostfixMaildrop, "Please use the :class:`insights.parsers.ls.LSlan` instead.", "3.5.0")
        super(LsVarSpoolPostfixMaildrop, self).__init__(*args, **kwargs)



import doctest

from insights.parsers import neutron_ml2_conf
from insights.parsers.neutron_ml2_conf import NeutronMl2Conf
from insights.tests import context_wrap

neutron_ml2_content = """
[DEFAULT]
verbose=True
debug=False
[ml2]
type_drivers=vxlan,vlan,flat,gre
tenant_network_types=vxlan
mechanism_drivers=openvswitch
extension_drivers=qos,port_security
path_mtu=0
overlay_ip_version=4
[ml2_type_flat]
flat_networks=datacentre
[ml2_type_geneve]
[ml2_type_gre]
tunnel_id_ranges=1:4094
[ml2_type_vlan]
network_vlan_ranges=datacentre:1:1000
[ml2_type_vxlan]
vni_ranges=1:4094
vxlan_group=224.0.0.1
[securitygroup]
firewall_driver=iptables_hybrid
"""


def test_neutron_ml2_conf():
    result = NeutronMl2Conf(context_wrap(neutron_ml2_content))
    assert result.get("ml2_type_flat", "flat_networks") == "datacentre"
    assert result.get("ml2", "tenant_network_types") == "vxlan"
    assert result.get("ml2", "mechanism_drivers") == "openvswitch"
    assert result.get("securitygroup", "firewall_driver") == "iptables_hybrid"


def test_neutron_ml2_conf_docs():
    failed, total = doctest.testmod(
        neutron_ml2_conf,
        globs={
            'neutron_ml2_conf': NeutronMl2Conf(context_wrap(neutron_ml2_content)),
        }
    )
    assert failed == 0



import json
import os

from tempfile import mkdtemp

from insights.core import dr
from insights.core.exceptions import ContentException
from insights.core.plugins import component, datasource, make_info, rule
from insights.core.serde import Hydration, deserializer, marshal, serializer, unmarshal
from insights.core.spec_factory import RegistryPoint, SpecSet
from insights.util import fs


class Foo(object):
    def __init__(self):
        self.a = 1
        self.b = 2


class Doo(object):
    def __init__(self, c):
        if c > 5:
            self.a = 1
            self.b = 2
        else:
            self.a = c
            self.b = 2


@component()
def thing():
    return Foo()


@serializer(Foo)
def serialize_foo(obj, root=None):
    return {"a": obj.a, "b": obj.b}


@serializer(Doo)
def serialize_doo(obj, root=None):
    if obj.a > 1:
        raise Exception('errors' + str(obj.a))
    return {"a": obj.a, "b": obj.b}


@deserializer(Foo)
def deserialize_foo(_type, data, root=None):
    foo = _type.__new__(_type)
    foo.a = data.get("a")
    foo.b = data.get("b")
    return foo


class Specs(SpecSet):
    the_data = RegistryPoint()


class TestSpecs(Specs):
    @datasource()
    def the_data(broker):
        raise ContentException('Fake Datasource')


@rule(Specs.the_data)
def report(dt):
    return make_info('INFO_1')


def test_marshal():
    broker = dr.Broker()
    foo = Foo()
    broker[thing] = foo
    data, errors = marshal(thing, broker)
    assert data
    assert not errors
    d = data["object"]
    assert d["a"] == 1
    assert d["b"] == 2


def test_marshal_with_errors():
    broker = dr.Broker()
    doo = Doo(2)
    broker[thing] = doo
    data, errors = marshal(thing, broker)
    assert not data
    assert isinstance(errors, str)

    # one raises error, one has result
    broker = dr.Broker()
    objs = [Doo(4), Doo(6)]
    broker[thing] = objs
    data, errors = marshal(thing, broker)
    assert data
    assert errors
    assert len(data) == 1
    assert len(errors) == 1
    d = data[0]["object"]
    assert d["a"] == 1
    assert d["b"] == 2

    # all raises error, no results
    broker = dr.Broker()
    objs = [Doo(4), Doo(3)]
    broker[thing] = objs
    data, errors = marshal(thing, broker)
    assert not data
    assert errors
    assert len(errors) == 2

    # all have resutls, no errors
    broker = dr.Broker()
    objs = [Doo(8), Doo(6)]
    broker[thing] = objs
    data, errors = marshal(thing, broker)
    assert data
    assert not errors
    assert len(data) == 2
    for item in data:
        d = item["object"]
        assert d["a"] == 1
        assert d["b"] == 2


def test_marshal_with_errors_with_pool():
    try:
        from concurrent.futures import ThreadPoolExecutor
        with ThreadPoolExecutor(thread_name_prefix="insights-collector-pool", max_workers=5) as pool:
            # one raises error, one has result
            broker = dr.Broker()
            objs = [Doo(4), Doo(6)]
            broker[thing] = objs
            data, errors = marshal(thing, broker, pool=pool)
            assert data
            assert errors
            assert len(data) == 1
            assert len(errors) == 1
            d = data[0]["object"]
            assert d["a"] == 1
            assert d["b"] == 2

            # all raises error, no results
            broker = dr.Broker()
            objs = [Doo(4), Doo(3)]
            broker[thing] = objs
            data, errors = marshal(thing, broker, pool=pool)
            assert not data
            assert errors
            assert len(errors) == 2

            # all have resutls, no errors
            broker = dr.Broker()
            objs = [Doo(8), Doo(6)]
            broker[thing] = objs
            data, errors = marshal(thing, broker, pool=pool)
            assert data
            assert not errors
            assert len(data) == 2
            for item in data:
                d = item["object"]
                assert d["a"] == 1
                assert d["b"] == 2
    except ImportError:
        pass


def test_unmarshal():
    d = {
        "type": "insights.tests.test_serde.Foo",
        "object": {"a": 1, "b": 2}
    }
    foo = unmarshal(d)
    assert foo is not None
    assert foo.a == 1
    assert foo.b == 2


def test_hydrate_one():
    raw = {
        "name": dr.get_name(thing),
        "exec_time": 0.25,
        "ser_time": 0.05,
        "results": {
            "type": "insights.tests.test_serde.Foo",
            "object": {"a": 1, "b": 2}
        }
    }

    h = Hydration()
    key, result, exec_time, ser_time = h._hydrate_one(raw)
    assert key is thing
    assert isinstance(result, Foo)
    assert exec_time == 0.25
    assert ser_time == 0.05


def test_hydrate_one_multiple_results():
    raw = {
        "name": dr.get_name(thing),
        "exec_time": 0.5,
        "ser_time": 0.1,
        "results": [
            {
                "type": "insights.tests.test_serde.Foo",
                "object": {"a": 1, "b": 2}
            },
            {
                "type": "insights.tests.test_serde.Foo",
                "object": {"a": 3, "b": 4}
            },
        ]
    }

    h = Hydration()
    key, result, exec_time, ser_time = h._hydrate_one(raw)
    assert key is thing
    assert len(result) == 2
    assert exec_time == 0.5
    assert ser_time == 0.1
    assert result[0].a == 1
    assert result[0].b == 2
    assert result[1].a == 3
    assert result[1].b == 4


def test_round_trip():
    tmp_path = mkdtemp()
    try:
        h = Hydration(tmp_path)

        broker = dr.Broker()
        broker[thing] = Foo()
        broker.exec_times[thing] = 0.5
        h.dehydrate(thing, broker)
        fn = ".".join([dr.get_name(thing), h.ser_name])
        assert os.path.exists(os.path.join(h.meta_data, fn))

        broker = h.hydrate()
        assert thing in broker
        assert broker.exec_times[thing] >= 0.5
        foo = broker[thing]
        assert foo.a == 1
        assert foo.b == 2
    finally:
        pass
        if os.path.exists(tmp_path):
            fs.remove(tmp_path)


def test_dehydrate():
    broker = dr.run(report)
    exc = next(iter(broker.tracebacks))
    tb = broker.tracebacks[exc]

    tmp_path = mkdtemp()
    spec_the_data = Specs.the_data
    try:
        h = Hydration(tmp_path)
        h.dehydrate(spec_the_data, broker)
        fn = ".".join([dr.get_name(spec_the_data), h.ser_name])
        meta_data_json = os.path.join(h.meta_data, fn)
        assert os.path.exists(meta_data_json)
        with open(meta_data_json, 'r') as fp:
            ret = json.load(fp)
            assert "errors" in ret
            assert ret["errors"][0] == tb
            assert "Traceback" in tb
            assert "Fake Datasource" in tb
    finally:
        fs.remove(tmp_path)



"""
ProcKeyUsers - File ``/proc/key-users``
=======================================

This parser reads the content of ``/proc/key-users``.

"""
from insights.core import Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.proc_keyusers)
class ProcKeyUsers(Parser, list):
    """
    Class ``ProcKeyUsers`` parses the content of the ``/proc/key-users`` file.

    This file lists various information for each user ID that has at least
    one key on the system. The fields in each line are in format::

        uid:    usage nkeys/nikeys qnkeys/maxkeys qnbytes/maxbytes

    The fields shown in each line will be parsed as follows::

        uid (string):   The user ID.
        usage (int):    This is a kernel-internal usage count for the
                        kernel structure used to record key users.
        nkeys (int):    The total number of keys owned by the user.
        nikeys (int):   The number of those keys that have been instantiated.

        qnkeys (int):   The number of keys owned by the user.
        maxkeys (int):  The maximum number of keys that the user may own.

        qnbytes (int):  The number of bytes consumed in payloads of the
                        keys owned by this user.
        maxbytes (int): The upper limit on the number of bytes in key
                        payloads for that user.

    Sample data::

            0:   106 105/105 95/1000000 1909/25000000
          862:     4 4/4 4/200 40/20000
          980:     9 9/9 9/200 138/20000
          983:     1 1/1 1/200 9/20000
         5090:   192 192/192 192/200 990/20000
         7502:     8 8/8 8/200 66/20000
         7563:     4 4/4 4/200 46/20000
         7637:     4 4/4 4/200 46/20000

    Examples:
        >>> type(proc_keyusers)
        <class 'insights.parsers.proc_keyusers.ProcKeyUsers'>
        >>> proc_keyusers[4]['uid']
        '5090'
        >>> proc_keyusers[4]['qnkeys']
        192
        >>> proc_keyusers[4]['maxkeys']
        200
        >>> proc_keyusers[4]['usage']
        192
        >>> len(proc_keyusers)
        8
    """

    def parse_content(self, content):

        fields = tuple(["usage", "nkeys", "nikeys", "qnkeys", "maxkeys", "qnbytes", "maxbytes"])

        for line in content:

            spl_colon = [_.strip() for _ in line.split(':')]
            value_pairs = spl_colon[-1].split()

            if not (len(spl_colon) == 2 and len(value_pairs) == 4 and spl_colon[0].isdigit()):
                raise ParseException("Unparsable line: {0}".format(line))

            try:
                field_values = [int(v) for vp in value_pairs for v in vp.split('/', 1)]
            except ValueError:
                raise ParseException("Unparsable line: not int value: {0}".format(line))

            if not len(field_values) == 7:
                raise ParseException("Unparsable line: unexpected value count: {0}".format(line))

            parsed_line = dict(zip(fields, field_values))
            parsed_line['uid'] = spl_colon[0]
            self.append(parsed_line)

        if len(self) == 0:
            raise SkipComponent



import doctest

from insights.parsr.query import first, last
from insights.parsers import httpd_conf
from insights.tests import context_wrap

HTTPD_CONF_1 = """
ServerRoot "/etc/httpd"
<Directory />
    Options FollowSymLinks
    AllowOverride None
</Directory>

SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

NSSProtocol SSLV3 TLSV1.0
#NSSProtocol ALL

# prefork MPM
 <IfModule prefork.c>
StartServers       8
MinSpareServers    5
MaxSpareServers   20
ServerLimit      256
MaxClients       256
MaxRequestsPerChild  200
 </IfModule>

# worker MPM
<IfModule worker.c>
StartServers         4
MaxClients         300
MinSpareThreads     25
MaxSpareThreads     75
ThreadsPerChild     25
MaxRequestsPerChild  0
</IfModule>
LoadModule auth_basic_module modules/mod_auth_basic.so
LoadModule auth_digest_module modules/mod_auth_digest.so
""".strip()

HTTPD_CONF_PATH = "/etc/httpd/conf/httpd.conf"
HTTPD_CONF_D_PATH = "/etc/httpd/conf.d/default.conf"

HTTPD_CONF_D_1 = """
SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

#SSLCipherSuite ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW
SSLCipherSuite ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW

# MaxClients: maximum number of server processes allowed to start
   MaxClients
""".strip()

HTTPD_CONF_SPLIT = '''
LogLevel warn
IncludeOptional conf.d/*.conf
EnableSendfile on
'''.strip()

HTTPD_CONF_MORE = '''
UserDir disable
UserDir enable bob
'''.strip()

HTTPD_CONF_NEST_1 = """
<VirtualHost 192.0.2.1> 
    <Directory /var/www/example>
        Options FollowSymLinks
        AllowOverride None
    </Directory>
    <IfModule mod_php4.c>
        php_admin_flag safe_mode Off
        php_admin_value register_globals    0
        php_value magic_quotes_gpc  0
        php_value magic_quotes_runtime  0
        php_value allow_call_time_pass_reference 0
    </IfModule>
    DirectoryIndex index.php
    <IfModule mod_rewrite.c>
        RewriteEngine On
        RewriteRule .* /index.php
    </IfModule>
    <IfModule mod_rewrite.c>
        RewriteEngine Off
    </IfModule>
    DocumentRoot /var/www/example
    ServerName www.example.com
    ServerAlias admin.example.com
</VirtualHost>
""".strip()  # noqa W291

HTTPD_CONF_NO_NAME_SEC = """
<RequireAll>
    AuthName "NAME Access"
    Require valid-user
</RequireAll>
""".strip()

HTTPD_CONF_DOC = '''
ServerRoot "/etc/httpd"
LoadModule auth_basic_module modules/mod_auth_basic.so
LoadModule auth_digest_module modules/mod_auth_digest.so

<Directory />
    Options FollowSymLinks
    AllowOverride None
</Directory>

<IfModule mod_mime_magic.c>
#   MIMEMagicFile /usr/share/magic.mime
    MIMEMagicFile conf/magic
</IfModule>

ErrorLog "|/usr/sbin/httplog -z /var/log/httpd/error_log.%Y-%m-%d"

SSLProtocol -ALL +SSLv3
#SSLProtocol all -SSLv2

NSSProtocol SSLV3 TLSV1.0
#NSSProtocol ALL

# prefork MPM
 <IfModule prefork.c>
StartServers       8
MinSpareServers    5
MaxSpareServers   20
ServerLimit      256
MaxClients       256
MaxRequestsPerChild  200
 </IfModule>

# worker MPM
<IfModule worker.c>
StartServers         4
MaxClients         300
MinSpareThreads     25
MaxSpareThreads     75
ThreadsPerChild     25
MaxRequestsPerChild  0
</IfModule>
'''.strip()


def test_get_httpd_conf_nest_1():
    context = context_wrap(HTTPD_CONF_NEST_1, path=HTTPD_CONF_PATH)
    result = httpd_conf.HttpdConf(context)
    assert result["VirtualHost", "192.0.2.1"]["IfModule", "mod_php4.c"]['php_admin_flag'][last].value == "safe_mode Off"
    assert result["VirtualHost", "192.0.2.1"]["IfModule", "mod_rewrite.c"]['RewriteEngine'][last].value is False
    assert result["VirtualHost", "192.0.2.1"]["IfModule", "mod_rewrite.c"]['RewriteRule'][last].value == ".* /index.php"
    assert result["VirtualHost", "192.0.2.1"]['ServerName'][last].value == "www.example.com"


def test_get_httpd_conf_1():
    context = context_wrap(HTTPD_CONF_1, path=HTTPD_CONF_PATH)
    result = httpd_conf.HttpdConf(context)

    assert "SSLCipherSuite" not in result
    assert result['ServerRoot'][first].value == '/etc/httpd'
    assert result["NSSProtocol"][first].value == "SSLV3 TLSV1.0"
    assert result["IfModule", "prefork.c"]["MaxClients"][last].value == 256
    assert result["IfModule", "worker.c"]["MaxClients"][last].value == 300
    assert result.file_path == HTTPD_CONF_PATH
    assert 'ThreadsPerChild' not in result['IfModule', 'prefork.c']
    assert result['IfModule', 'prefork.c']['MaxRequestsPerChild'][last].value == 200
    assert result.file_name == "httpd.conf"
    assert result['LoadModule'][first].value == 'auth_basic_module modules/mod_auth_basic.so'
    assert result['LoadModule'][last].value == 'auth_digest_module modules/mod_auth_digest.so'
    assert result['Directory', '/']['Options'][last].value == 'FollowSymLinks'


def test_get_httpd_conf_2():
    context = context_wrap(HTTPD_CONF_D_1, path=HTTPD_CONF_D_PATH)
    result = httpd_conf.HttpdConf(context)

    except_SSLC = 'ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW'
    assert result["SSLCipherSuite"][last].value == except_SSLC
    assert "NSSProtocol" not in result
    assert result.file_path == HTTPD_CONF_D_PATH
    assert result.file_name == "default.conf"
    assert result["SSLProtocol"][last].value == '-ALL +SSLv3'
    assert result["SSLProtocol"][last].line == 'SSLProtocol -ALL +SSLv3'


def test_main_config_splitting():
    context = context_wrap(HTTPD_CONF_SPLIT, path=HTTPD_CONF_PATH)
    result = httpd_conf.HttpdConf(context)

    assert result.file_path == HTTPD_CONF_PATH
    assert result.file_name == "httpd.conf"
    assert result['LogLevel'][-1].value == 'warn'
    assert result['LogLevel'][-1].line == 'LogLevel warn'
    assert result['EnableSendfile'][-1].value is True


def test_multiple_values_for_directive():
    context = context_wrap(HTTPD_CONF_MORE, path=HTTPD_CONF_PATH)
    result = httpd_conf.HttpdConf(context)

    assert result.file_path == HTTPD_CONF_PATH
    assert result.file_name == "httpd.conf"
    assert len(result['UserDir']) == 2
    assert result['UserDir'][0].value == 'disable'
    assert result['UserDir'][1].value == 'enable bob'


def test_no_name_section():
    context = context_wrap(HTTPD_CONF_NO_NAME_SEC, path=HTTPD_CONF_PATH)
    result = httpd_conf.HttpdConf(context)

    assert result["RequireAll"]["AuthName"][last].value == "NAME Access"
    assert result["RequireAll"]["Require"][last].value == "valid-user"


def test_doc():
    env = {
            'httpd_conf.HttpdConf': httpd_conf.HttpdConf,
            'httpd_conf': httpd_conf.HttpdConf(context_wrap(HTTPD_CONF_DOC, path='/path')),
          }
    failed, total = doctest.testmod(httpd_conf, globs=env)
    assert failed == 0



"""
IrisList - Command ``/usr/bin/iris list``
=========================================

This parser reads the output of ``/usr/bin/iris list``.
"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.iris_list)
class IrisList(CommandParser, dict):
    """
    Parse the output of the ``/usr/bin/iris list`` command.

    Sample Input::

        Configuration 'IRIS'   (default)
            directory:    /intersystems
            versionid:    2023.1.0.235.1com
            datadir:      /intersystems
            conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
            status:       running, since Tue Jun 27 01:55:25 2023
            state:        ok
            product:      InterSystems IRIS

    Examples:
        >>> iris_info['name']
        'IRIS'
        >>> iris_info['status']
        'running, since Tue Jun 27 01:55:25 2023'
        >>> iris_info.is_running
        True
    """

    def parse_content(self, content):
        for line in content:
            if not line.strip():
                continue
            if line.strip().startswith('Configuration'):
                self['name'] = line.split()[1].strip('\'"')
            elif ":" in line:
                key, value = line.split(":", 1)
                self[key.strip()] = value.strip()

        if len(self) == 0:
            raise SkipComponent("The result is empty")

    @property
    def is_running(self):
        """Return True when the iris instance is running, and False when it is down"""
        return self.get('status', "").startswith('running')



import pytest

from insights.combiners.hostname import Hostname
from insights.combiners.sap import Sap
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.hostname import Hostname as HostnameParser
from insights.parsers.saphostctrl import SAPHostCtrlInstances
from insights.specs.datasources.sap import (LocalSpecs, ld_library_path_of_user, sap_hana_sid,
                                            sap_hana_sid_SID_nr, sap_sid)
from insights.tests.combiners.test_sap import (HOSTNAME1, SAPHOSTCTRL_HOSTINSTANCES_GOOD,
                                               SAPHOSTCTRL_HOSTINSTANCES_R_CASE)
from insights.tests import context_wrap

SAPHOSTCTRL_HOSTINSTANCES = '''
*********************************************************
 SID , String , RH1
 SystemNumber , String , 01
 InstanceName , String , ASCS01
 InstanceType , String , ABAP Central Services
 Hostname , String , vm37-39
 FullQualifiedHostname , String , vm37-39.pek2.com
 IPAddress , String , 10.72.37.39
 SapVersionInfo , String , 745, patch 100, changelist 1652052
*********************************************************
 SID , String , RH1
 SystemNumber , String , 00
 InstanceName , String , D00
 InstanceType , String , ABAP Dialog Instance
 Hostname , String , vm37-39
 FullQualifiedHostname , String , vm37-39.pek2.com
 IPAddress , String , 10.72.37.39
 SapVersionInfo , String , 745, patch 100, changelist 1652052
*********************************************************
 SID , String , SR1
 SystemNumber , String , 02
 InstanceName , String , HDB02
 InstanceType , String , HANA
 Hostname , String , vm37-39
 FullQualifiedHostname , String , vm37-39.pek2.com
 IPAddress , String , 10.72.37.39
 SapVersionInfo , String , 749, patch 418, changelist 1816226
*********************************************************
 SID , String , RH2
 SystemNumber , String , 04
 InstanceName , String , ASCS04
 InstanceType , String , ABAP Central Services
 Hostname , String , vm37-39
 FullQualifiedHostname , String , vm37-39.pek2.com
 IPAddress , String , 10.72.37.39
 SapVersionInfo , String , 745, patch 100, changelist 1652052
*********************************************************
 SID , String , RH2
 SystemNumber , String , 03
 InstanceName , String , D03
 InstanceType , String , ABAP Dialog Instance
 Hostname , String , vm37-39
 FullQualifiedHostname , String , vm37-39.pek2.com
 IPAddress , String , 10.72.37.39
 SapVersionInfo , String , 745, patch 100, changelist 1652052
'''.strip()

RH1ADM_ENV = '''
TERM=screen-256color
HOME=/home/rh1adm
SHELL=/bin/csh
USER=rh1adm
LOGNAME=rh1adm
PATH=/sapdb/clients/RH1/bin:/sapdb/programs/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/usr/sap/RH1/SYS/exe/run:/home/rh1adm:.
XDG_SESSION_ID=6682
HOSTTYPE=x86_64-linux
VENDOR=unknown
OSTYPE=linux
MACHTYPE=x86_64
SHLVL=1
PWD=/home/rh1adm
GROUP=sapsys
HOST=vm37-39
REMOTEHOST=10.66.136.143
MAIL=/var/spool/mail/rh1adm
HOSTNAME=vm37-39
LANG=en_US.UTF-8
LESSOPEN=||/usr/bin/lesspipe.sh %s
SAPSYSTEMNAME=RH1
DIR_LIBRARY=/usr/sap/RH1/SYS/exe/run
RSEC_SSFS_DATAPATH=/usr/sap/RH1/SYS/global/security/rsecssfs/data
RSEC_SSFS_KEYPATH=/usr/sap/RH1/SYS/global/security/rsecssfs/key
rsdb_ssfs_connect=0
LD_LIBRARY_PATH=/usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib
dbms_type=ADA
'''.strip()

RH2ADM_ENV = '''
TERM=screen-256color
HOME=/home/rh2adm
SHELL=/bin/csh
USER=rh2adm
LOGNAME=rh2adm
PATH=/sapdb/clients/RH2/bin:/sapdb/programs/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/usr/sap/RH2/SYS/exe/uc/linuxx86_64:/usr/sap/RH2/SYS/exe/run:/home/rh2adm:.
XDG_SESSION_ID=6682
HOSTTYPE=x86_64-linux
VENDOR=unknown
OSTYPE=linux
MACHTYPE=x86_64
SHLVL=1
PWD=/home/rh2adm
GROUP=sapsys
HOST=vm37-39
REMOTEHOST=10.66.136.143
MAIL=/var/spool/mail/rh2adm
HOSTNAME=vm37-39
LANG=en_US.UTF-8
LESSOPEN=||/usr/bin/lesspipe.sh %s
SAPSYSTEMNAME=RH2
DIR_LIBRARY=/usr/sap/RH2/SYS/exe/run
RSEC_SSFS_DATAPATH=/usr/sap/RH2/SYS/global/security/rsecssfs/data
RSEC_SSFS_KEYPATH=/usr/sap/RH2/SYS/global/security/rsecssfs/key
rsdb_ssfs_connect=0
LD_LIBRARY_PATH=/usr/sap/RH2/SYS/exe/run:/usr/sap/RH2/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH2/lib
dbms_type=ADA
'''.strip()

HOSTNAME = 'vm37-39.pek2.com'


class FakeContext(object):
    def shell_out(self, cmd, split=True, timeout=None, keep_rc=False, env=None, signum=None):
        tmp_cmd = cmd.strip().split()
        if 'rh1adm' in tmp_cmd[2]:
            return 0, RH1ADM_ENV.splitlines()
        if 'rh2adm' in tmp_cmd[2]:
            return 0, RH2ADM_ENV.splitlines()
        return -1, []


def test_hana_instance_skip():
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_R_CASE))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    with pytest.raises(SkipComponent):
        LocalSpecs.sap_hana_instance(broker)


def test_sid():
    # Good
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    result = sap_sid(broker)
    assert result is not None
    assert isinstance(result, list)
    assert result == sorted(set(v.sid.lower() for v in sap.values()))


def test_hana_sid():
    # Good
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    broker.update({LocalSpecs.sap_hana_instance: LocalSpecs.sap_hana_instance(broker)})
    result = sap_hana_sid(broker)
    assert result is not None
    assert isinstance(result, list)
    assert result == list(set(v.sid.lower() for v in sap.values() if v.type == 'HDB'))

    # Bad
    broker.update({LocalSpecs.sap_hana_instance: []})
    with pytest.raises(SkipComponent):
        sap_hana_sid(broker)


def test_hana_sid_SID_nr():
    # Good
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    broker.update({LocalSpecs.sap_hana_instance: LocalSpecs.sap_hana_instance(broker)})
    result = sap_hana_sid_SID_nr(broker)
    assert result is not None
    assert isinstance(result, list)
    assert result == list((v.sid.lower(), v.sid, v.number) for v in sap.values() if v.type == 'HDB')

    # Bad
    broker.update({LocalSpecs.sap_hana_instance: []})
    with pytest.raises(SkipComponent):
        sap_hana_sid_SID_nr(broker)


def test_ld_library_path_of_user():
    # Good
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap, HostContext: FakeContext()}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    broker.update({sap_sid: sap_sid(broker)})
    result = ld_library_path_of_user(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    assert sorted(result.content) == [
        'rh1adm /usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib',
        'rh2adm /usr/sap/RH2/SYS/exe/run:/usr/sap/RH2/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH2/lib',
    ]
    assert result.relative_path == 'insights_commands/echo_user_LD_LIBRARY_PATH'

    # Bad
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_GOOD))
    hn = Hostname(HostnameParser(context_wrap(HOSTNAME1)), None, None, None)
    sap = Sap(hn, inst, None)
    broker = {Sap: sap, HostContext: FakeContext()}
    broker.update({LocalSpecs.sap_instance: LocalSpecs.sap_instance(broker)})
    broker.update({sap_sid: sap_sid(broker)})
    with pytest.raises(SkipComponent):
        result = ld_library_path_of_user(broker)
        assert result is None



"""
CronDailyRhsmd - file ``/etc/cron.daily/rhsmd``
===============================================
"""

from insights.core import Scannable
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.cron_daily_rhsmd)
class CronDailyRhsmd(Scannable):
    """
    Parse the ``/etc/cron.daily/rhsmd`` file.

    Sample input::

        config=$(grep -E "^processTimeout" /etc/rhsm/rhsm.conf | grep -Po "[0-9]+")
        rhsmd_timeout=$config
        abc=$config

    .. note::
        Please refer to its super-class :py:class:`insights.core.Scannable`
        for full usage.

    Examples:

        >>> # CronDailyRhsmd.collect('config_lines', lambda n: n if "$config" in n else "")
        >>> # CronDailyRhsmd.any('one_config_line', lambda n: n if "$config" in n else "")
        >>> rhsmd.config_lines
        ['rhsmd_timeout=$config', 'abc=$config']
        >>> rhsmd.one_config_line
        'rhsmd_timeout=$config'
    """
    pass



import doctest

from insights.parsers import ls_var_lib_rsyslog
from insights.tests import context_wrap


LS_VAR_LIB_RSYSLOG_1 = """
total 4
-rw-------. 1 root root system_u:object_r:syslogd_var_lib_t:s0 127 Nov 30 03:40 imjournal.state
"""

LS_VAR_LIB_RSYSLOG_2 = """
total 4
-rw-------. root root system_u:object_r:syslogd_var_lib_t:s0 imjournal.state
"""


def test_ls_var_lib_rsyslog():
    rsyslog_obj = ls_var_lib_rsyslog.LsVarLibRsyslog(context_wrap(LS_VAR_LIB_RSYSLOG_2, path="insights_commands/ls_-lZ_.var.lib.rsyslog"))
    assert rsyslog_obj.files_of('/var/lib/rsyslog') == ['imjournal.state']
    journal_obj = rsyslog_obj.dir_entry('/var/lib/rsyslog', 'imjournal.state')
    assert journal_obj is not None
    assert journal_obj['se_type'] == 'syslogd_var_lib_t'
    assert journal_obj['owner'] == 'root'


def test_ls_var_lib_pcp_doc_examples():
    env = {
        'rsyslog_obj': ls_var_lib_rsyslog.LsVarLibRsyslog(context_wrap(LS_VAR_LIB_RSYSLOG_1, path="insights_commands/ls_-lZ_.var.lib.rsyslog")),
    }
    failed, total = doctest.testmod(ls_var_lib_rsyslog, globs=env)
    assert failed == 0



import doctest

from insights.parsers import ls_dev
from insights.parsers.ls_dev import LsDev
from insights.tests import context_wrap

LS_DEV = """
/dev:
total 3
brw-rw----.  1 0  6 253,   0 Aug  4 16:56 dm-0
brw-rw----.  1 0  6 253,   1 Aug  4 16:56 dm-1
brw-rw----.  1 0  6 253,  10 Aug  4 16:56 dm-10
crw-rw-rw-.  1 0  5   5,   2 Aug  5  2016 ptmx
drwxr-xr-x.  2 0  0        0 Aug  4 16:56 pts
lrwxrwxrwx.  1 0  0       25 Oct 25 14:48 initctl -> /run/systemd/initctl/fifo

/dev/rhel:
total 0
drwxr-xr-x.  2 0 0  100 Jul 25 10:00 .
drwxr-xr-x. 23 0 0 3720 Jul 25 12:43 ..
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 root -> ../dm-0
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 swap -> ../dm-1

/dev/mapper:
total 0
drwxr-xr-x.  2 0 0     140 Jul 25 10:00 .
drwxr-xr-x. 23 0 0    3720 Jul 25 12:43 ..
crw-------.  1 0 0 10, 236 Jul 25 10:00 control
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 docker-253:0-1443032-pool -> ../dm-3
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-home -> ../dm-2
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-root -> ../dm-0
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-swap -> ../dm-1
"""


def test_ls_dev():
    lsdev = LsDev(context_wrap(LS_DEV))
    assert lsdev.listing_of("/dev/rhel") == {
        'home': {'group': '0', 'name': 'home', 'links': 1, 'perms': 'rwxrwxrwx.',
                 'raw_entry': 'lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2',
                 'owner': '0', 'link': '../dm-2', 'date': 'Jul 25 10:00',
                 'type': 'l', 'size': 7, 'dir': '/dev/rhel'},
        'root': {'group': '0', 'name': 'root', 'links': 1, 'perms': 'rwxrwxrwx.',
                 'raw_entry': 'lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 root -> ../dm-0',
                 'owner': '0', 'link': '../dm-0', 'date': 'Jul 25 10:00',
                 'type': 'l', 'size': 7, 'dir': '/dev/rhel'},
        'swap': {'group': '0', 'name': 'swap', 'links': 1, 'perms': 'rwxrwxrwx.',
                 'raw_entry': 'lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 swap -> ../dm-1',
                 'owner': '0', 'link': '../dm-1', 'date': 'Jul 25 10:00',
                 'type': 'l', 'size': 7, 'dir': '/dev/rhel'},
        '..': {'group': '0', 'name': '..', 'links': 23, 'perms': 'rwxr-xr-x.',
               'raw_entry': 'drwxr-xr-x. 23 0 0 3720 Jul 25 12:43 ..',
               'owner': '0', 'date': 'Jul 25 12:43', 'type': 'd', 'size': 3720,
               'dir': '/dev/rhel'},
        '.': {'group': '0', 'name': '.', 'links': 2, 'perms': 'rwxr-xr-x.',
              'raw_entry': 'drwxr-xr-x.  2 0 0  100 Jul 25 10:00 .', 'owner': '0',
              'date': 'Jul 25 10:00', 'type': 'd', 'size': 100, 'dir': '/dev/rhel'}}
    expected = ['docker-253:0-1443032-pool', 'rhel-home', 'rhel-root', 'rhel-swap']
    actual = lsdev.listings.get("/dev/mapper")['files']
    assert actual == expected

    assert lsdev.listings.get("/dev/mapper")['entries']['rhel-home']['link'] == "../dm-2"


def test_doc_examples():
    env = {'ls_dev': LsDev(context_wrap(LS_DEV))}
    failed, total = doctest.testmod(ls_dev, globs=env)
    assert failed == 0



import json
import pytest
import shutil
import tempfile

from mock.mock import Mock

from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.ps import ps_eo_cmd, LocalSpecs, jboss_runtime_versions

PS_DATA = """
PID COMMAND
  1 /usr/lib/systemd/systemd --switched-root --system --deserialize 31
  2 [kthreadd]
  3 [rcu_gp]
  4 [rcu_par_gp]
  6 [kworker/0:0H-events_highpri]
  9 [mm_percpu_wq]
 10 [rcu_tasks_kthre]
 11 /usr/bin/python3 /home/user1/python_app.py
 12 [kworker/u16:0-kcryptd/253:0]
"""

PS_EXPECTED = """
PID COMMAND
1 /usr/lib/systemd/systemd
2 [kthreadd]
3 [rcu_gp]
4 [rcu_par_gp]
6 [kworker/0:0H-events_highpri]
9 [mm_percpu_wq]
10 [rcu_tasks_kthre]
11 /usr/bin/python3
12 [kworker/u16:0-kcryptd/253:0]
"""

PS_BAD = "Command not found"

PS_EMPTY = """
PID COMMAND
"""
jboss_home_1 = tempfile.mkdtemp(prefix='insights_test')
jboss_home_2 = tempfile.mkdtemp(prefix='insights_test')
PS_JBOSS_VERSION = """
    PID COMMAND
    1 /usr/lib/systemd/systemd --switched-root --system --deserialize 17
    2 [kthreadd]
    3 [rcu_gp]
    8686 java -D[Standalone] -server -verbose:gc -Xloggc:/opt/jboss-datagrid-7.3.0-server/standalone/log/gc.log -XX:+PrintGCDetails -XX:+UseGCLogFileRotation -XX:NumberOfGCLogFiles=5 -XX:GCLogFileSize=3M -XX:-TraceClassUnloading -Xms64m -Xmx512m -XX:MetaspaceSize=96M -XX:MaxMetaspaceSize=256m -Djava.net.preferIPv4Stack=true -Djboss.modules.system.pkgs=org.jboss.byteman -Djava.awt.headless=true -Dorg.jboss.boot.log.file=/opt/jboss-datagrid-7.3.0-server/standalone/log/server.log -Dlogging.configuration=file:/opt/jboss-datagrid-7.3.0-server/standalone/configuration/logging.properties -jar /opt/jboss-datagrid-7.3.0-server/jboss-modules.jar -mp /opt/jboss-datagrid-7.3.0-server/modules org.jboss.as.standalone -Djboss.home.dir={0} -Djboss.server.base.dir=/opt/jboss-datagrid-7.3.0-server/standalone
    8880 /usr/lib/jvm/java-1.8.0-oracle/bin/java -D[Process Controller] -server -Xms64m -Xmx512m -XX:MaxMetaspaceSize=256m -Djava.net.preferIPv4Stack=true -Djboss.modules.system.pkgs=org.jboss.byteman -Djava.awt.headless=true -Dorg.jboss.boot.log.file=/home/jboss/jboss-eap-7.1/domain/log/process-controller.log -Dlogging.configuration=file:/home/jboss/jboss-eap-7.1/domain/configuration/logging.properties -jar /home/jboss/jboss-eap-7.1/jboss-modules.jar -mp /home/jboss/jboss-eap-7.1/modules org.jboss.as.process-controller -jboss-home {1} -jvm /usr/lib/jvm/java-1.8.0-oracle/bin/java -mp /home/jboss/jboss-eap-7.1/modules -- -Dorg.jboss.boot.log.file=/home/jboss/jboss-eap-7.1/domain/log/host-controller.log -Dlogging.configuration=file:/home/jboss/jboss-eap-7.1/domain/configuration/logging.properties -server
""".format(jboss_home_1, jboss_home_2)

RELATIVE_PATH = 'insights_commands/ps_eo_cmd'
JBOSS_VERSION_PATH = 'insights_commands/jboss_versions'


def test_ps_eo_cmd():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_DATA.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    result = ps_eo_cmd(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=PS_EXPECTED.strip(), relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_ps_eo_cmd_bad():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_BAD.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    with pytest.raises(SkipComponent) as e:
        ps_eo_cmd(broker)
    assert e is not None


def test_ps_eo_cmd_empty():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_EMPTY.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    with pytest.raises(SkipComponent) as e:
        ps_eo_cmd(broker)
    assert e is not None


def test_jboss_versions():
    # create version.txt file
    with open(jboss_home_1 + '/version.txt', 'w') as j_v:
        j_v.write("Red Hat JBoss Enterprise Application Platform - Version 7.4.0.GA")
    with open(jboss_home_2 + '/version.txt', 'w') as j_v:
        j_v.write("Red Hat Data Grid - Version 8.3.1.GA")
    ps_eo_args = Mock()
    ps_eo_args.content = PS_JBOSS_VERSION.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    result = jboss_runtime_versions(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected_content = {jboss_home_1: 'Red Hat JBoss Enterprise Application Platform - Version 7.4.0.GA',
                        jboss_home_2: 'Red Hat Data Grid - Version 8.3.1.GA'}
    expected = DatasourceProvider(content=json.dumps(expected_content), relative_path=JBOSS_VERSION_PATH)
    result_dict = json.loads(result.content[0])
    assert result_dict == expected_content
    assert result.relative_path == expected.relative_path
    shutil.rmtree(jboss_home_1)
    shutil.rmtree(jboss_home_2)



from insights import make_fail, rule
from insights.parsers.uname import Uname


@rule(Uname)
def report(uname):
    if uname.fixed_by('2.6.32-431.11.2.el6', introduced_in='2.6.32-431.el6'):
        return make_fail("VULNERABLE_KERNEL", kernel=uname.kernel)



"""
LsVarLibPcp - command ``ls -la /var/lib/pcp``
=============================================

The ``ls -la /var/lib/pcp`` command provides information for the listing of the ``/var/lib/pcp`` directory.

The parsers class in this module uses base parser class
``CommandParser`` & ``FileListing`` to list files & directories.

Sample output of this command is::

    total 16
    drwxr-xr-x.  4 root root  33 Dec 15 08:12 .
    drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..
    drwxr-xr-x.  2 root root   6 Oct  3 09:37 pmcd
    drwxr-xr-x.  2 root root   6 Oct  3 09:37 pmie
    drwxrwxr-x.  4 root root 128 Aug 22 17:55 pmdas

Examples:

    >>> "pmdas" in ls_var_lib_pcp
    False
    >>> "/var/lib/pcp" in ls_var_lib_pcp
    True
    >>> ls_var_lib_pcp.dir_entry('/var/lib/pcp', 'pmdas')['type']
    'd'
"""


from insights.specs import Specs
from insights import CommandParser, parser
from insights.core import FileListing
from insights.util import deprecated


@parser(Specs.ls_var_lib_pcp)
class LsVarLibPcp(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSla` instead.

    Parses output of ``ls -la /var/lib/pcp`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarLibPcp, "Please use the :class:`insights.parsers.ls.LSla` instead.", "3.5.0")
        super(LsVarLibPcp, self).__init__(*args, **kwargs)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import docker_inspect
from insights.tests import context_wrap

DOCKER_CONTAINER_INSPECT = """
[
{
    "Id": "97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07",
    "Created": "2016-06-23T05:12:25.433469799Z",
    "Path": "/bin/bash",
    "Args": [],
    "State": {
        "Status": "running",
        "Running": true,
        "Paused": false,
        "Restarting": false,
        "OOMKilled": false,
        "Dead": false,
        "Pid": 15096,
        "ExitCode": 0,
        "Error": "",
        "StartedAt": "2016-06-23T05:37:56.925378831Z",
        "FinishedAt": "2016-06-23T05:33:02.012653984Z"
    },
    "Image": "882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2",
    "ResolvConfPath": "/var/lib/docker/containers/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07/resolv.conf",
    "HostnamePath": "/var/lib/docker/containers/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07/hostname",
    "HostsPath": "/var/lib/docker/containers/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07/hosts",
    "LogPath": "/var/lib/docker/containers/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07-json.log",
    "Name": "/hehe2",
    "RestartCount": 0,
    "Driver": "devicemapper",
    "ExecDriver": "native-0.2",
    "MountLabel": "system_u:object_r:svirt_sandbox_file_t:s0:c429,c690",
    "ProcessLabel": "system_u:system_r:svirt_lxc_net_t:s0:c429,c690",
    "AppArmorProfile": "",
    "ExecIDs": null,
    "HostConfig": {
        "Binds": null,
        "ContainerIDFile": "",
        "LxcConf": [],
        "Memory": 0,
        "MemoryReservation": 0,
        "MemorySwap": 0,
        "KernelMemory": 0,
        "CpuShares": 0,
        "CpuPeriod": 0,
        "CpusetCpus": "",
        "CpusetMems": "",
        "CpuQuota": 0,
        "BlkioWeight": 0,
        "OomKillDisable": false,
        "MemorySwappiness": -1,
        "Privileged": false,
        "PortBindings": {},
        "Links": null,
        "PublishAllPorts": false,
        "Dns": [],
        "DnsOptions": [],
        "DnsSearch": [],
        "ExtraHosts": null,
        "VolumesFrom": null,
        "Devices": [],
        "NetworkMode": "default",
        "IpcMode": "",
        "PidMode": "",
        "UTSMode": "",
        "CapAdd": null,
        "CapDrop": null,
        "GroupAdd": null,
        "RestartPolicy": {
            "Name": "no",
            "MaximumRetryCount": 0
        },
        "SecurityOpt": null,
        "ReadonlyRootfs": false,
        "Ulimits": null,
        "Sysctls": {},
        "LogConfig": {
            "Type": "json-file",
            "Config": {
                "max-file": "7",
                "max-size": "10m"
            }
        },
        "CgroupParent": "",
        "ConsoleSize": [
            0,
            0
        ],
        "VolumeDriver": "",
        "ShmSize": 67108864
    },
    "GraphDriver": {
        "Name": "devicemapper",
        "Data": {
            "DeviceId": "433",
            "DeviceName": "docker-253:0-71431059-97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07",
            "DeviceSize": "107374182400"
        }
    },
    "Mounts": [],
    "Config": {
        "Hostname": "97d7cd1a5d8f",
        "Domainname": "",
        "User": "root",
        "AttachStdin": true,
        "AttachStdout": true,
        "AttachStderr": true,
        "Tty": true,
        "OpenStdin": true,
        "StdinOnce": true,
        "Env": [
            "container=docker",
            "PKGM=yum",
            "PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin"
        ],
        "Cmd": [
            "/bin/bash"
        ],
        "Image": "rhel7_imagemagick",
        "Volumes": null,
        "WorkingDir": "",
        "Entrypoint": null,
        "OnBuild": null,
        "Labels": {
            "Architecture": "x86_64",
            "Authoritative_Registry": "registry.access.redhat.com",
            "BZComponent": "rhel-server-docker",
            "Build_Host": "rcm-img03.build.eng.bos.redhat.com",
            "Name": "rhel7/rhel",
            "Release": "61",
            "Vendor": "Red Hat, Inc.",
            "Version": "7.2",
            "io.projectatomic.Temporary": "true"
        },
        "StopSignal": "SIGTERM"
    },
    "NetworkSettings": {
        "Bridge": "",
        "SandboxID": "f1cce5397340364aff043879ff5bd7e2ce2fcc5b81cfb7fe1833ce7b57eb6cf8",
        "HairpinMode": false,
        "LinkLocalIPv6Address": "",
        "LinkLocalIPv6PrefixLen": 0,
        "Ports": {},
        "SandboxKey": "/var/run/docker/netns/f1cce5397340",
        "SecondaryIPAddresses": null,
        "SecondaryIPv6Addresses": null,
        "EndpointID": "59be4c94b2a1346eb0ec16472bc132e071d18733fd956c34b3b1defff9bba389",
        "Gateway": "172.17.0.1",
        "GlobalIPv6Address": "",
        "GlobalIPv6PrefixLen": 0,
        "IPAddress": "172.17.0.2",
        "IPPrefixLen": 16,
        "IPv6Gateway": "",
        "MacAddress": "02:42:ac:11:00:02",
        "Networks": {
            "bridge": {
                "EndpointID": "59be4c94b2a1346eb0ec16472bc132e071d18733fd956c34b3b1defff9bba389",
                "Gateway": "172.17.0.1",
                "IPAddress": "172.17.0.2",
                "IPPrefixLen": 16,
                "IPv6Gateway": "",
                "GlobalIPv6Address": "",
                "GlobalIPv6PrefixLen": 0,
                "MacAddress": "02:42:ac:11:00:02"
            }
        }
    }
}
]
""".splitlines()

DOCKER_IMAGE_INSPECT = """
[
{
    "Id": "882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2",
    "RepoTags": [
        "rhel7_imagemagick:latest"
    ],
    "RepoDigests": [],
    "Parent": "34c167d900afb820ecab622a214ce3207af80ec755c0dcb6165b425087ddbc3a",
    "Comment": "",
    "Created": "2016-06-23T03:39:15.068803433Z",
    "Container": "65410bf8809af52d2074c882917ea0651b119a91f460c1037bc99d4d5976532a",
    "ContainerConfig": {
        "Hostname": "cf3092658f01",
        "Domainname": "",
        "User": "root",
        "AttachStdin": false,
        "AttachStdout": false,
        "AttachStderr": false,
        "Tty": false,
        "OpenStdin": false,
        "StdinOnce": false,
        "Env": [
            "container=docker",
            "PKGM=yum",
            "PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin"
        ],
        "Cmd": [
            "/bin/sh",
            "-c",
            "yum install -y ImageMagick-6.7.8.9-10.el7"
        ],
        "Image": "34c167d900afb820ecab622a214ce3207af80ec755c0dcb6165b425087ddbc3a",
        "Volumes": null,
        "WorkingDir": "",
        "Entrypoint": null,
        "OnBuild": [],
        "Labels": {
            "Architecture": "x86_64",
            "Authoritative_Registry": "registry.access.redhat.com",
            "BZComponent": "rhel-server-docker",
            "Build_Host": "rcm-img03.build.eng.bos.redhat.com",
            "Name": "rhel7/rhel",
            "Release": "61",
            "Vendor": "Red Hat, Inc.",
            "Version": "7.2"
        }
    },
    "DockerVersion": "1.9.1",
    "Author": "",
    "Config": {
        "Hostname": "cf3092658f01",
        "Domainname": "",
        "User": "root",
        "AttachStdin": false,
        "AttachStdout": false,
        "AttachStderr": false,
        "Tty": false,
        "OpenStdin": false,
        "StdinOnce": false,
        "Env": [
            "container=docker",
            "PKGM=yum",
            "PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin"
        ],
        "Cmd": [
            "/usr/bin/bash"
        ],
        "Image": "34c167d900afb820ecab622a214ce3207af80ec755c0dcb6165b425087ddbc3a",
        "Volumes": null,
        "WorkingDir": "",
        "Entrypoint": null,
        "OnBuild": [],
        "Labels": {
            "Architecture": "x86_64",
            "Authoritative_Registry": "registry.access.redhat.com",
            "BZComponent": "rhel-server-docker",
            "Build_Host": "rcm-img03.build.eng.bos.redhat.com",
            "Name": "rhel7/rhel",
            "Release": "61",
            "Vendor": "Red Hat, Inc.",
            "Version": "7.2"
        }
    },
    "Architecture": "amd64",
    "Os": "linux",
    "Size": 580094174,
    "VirtualSize": 785437820,
    "GraphDriver": {
        "Name": "devicemapper",
        "Data": {
            "DeviceId": "431",
            "DeviceName": "docker-253:0-71431059-882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2",
            "DeviceSize": "107374182400"
        }
    }
}
]
""".splitlines()


DOCKER_CONTAINER_INSPECT_TRUNCATED = """
[
{
    "Id": "97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07",
    "Created": "2016-06-23T05:12:25.433469799Z",
    "Path": "/bin/bash",
"""


def test_docker_object_container_inspect():
    result = docker_inspect.DockerInspect(context_wrap(DOCKER_CONTAINER_INSPECT))
    assert result.get('Id') == "97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07"
    assert result.get('NetworkSettings').get('HairpinMode') is False
    assert result.get('Config').get('Env') == ['container=docker', 'PKGM=yum', 'PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin']
    assert result.get('GraphDriver').get('Data').get('DeviceSize') == '107374182400'


def test_docker_object_image_inspect():
    result = docker_inspect.DockerInspect(context_wrap(DOCKER_IMAGE_INSPECT))
    assert result.get('Id') == "882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2"
    assert result.get('Size') == 580094174
    assert result.get('Config').get('AttachStdin') is False
    assert result.get('RepoDigests') == []


def test_docker_container_inspect_truncated_input():
    with pytest.raises(SkipComponent):
        docker_inspect.DockerInspectContainer(context_wrap(DOCKER_CONTAINER_INSPECT_TRUNCATED))


def test_doc_test():
    dic = docker_inspect.DockerInspectContainer(context_wrap(DOCKER_CONTAINER_INSPECT))
    dii = docker_inspect.DockerInspectImage(context_wrap(DOCKER_IMAGE_INSPECT))
    env = {
            'container': dic,
            'image': dii,
    }
    failed, total = doctest.testmod(docker_inspect, globs=env)
    assert failed == 0



from insights.parsers import dockerinfo
from insights.tests import context_wrap


docker_info1 = """
Containers: 0
Images: 1
Server Version: 1.9.1
Storage Driver: devicemapper
 Pool Name: docker-253:0-10499681-pool
 Pool Blocksize: 65.54 kB
 Base Device Size: 107.4 GB
 Backing Filesystem:
 Data file: /dev/loop0
 Metadata file: /dev/loop1
 Data Space Used: 295.9 MB
 Data Space Total: 107.4 GB
 Data Space Available: 3.611 GB
 Metadata Space Used: 716.8 kB
 Metadata Space Total: 2.147 GB
 Metadata Space Available: 2.147 GB
 Udev Sync Supported: true
 Deferred Removal Enabled: false
 Deferred Deletion Enabled: false
 Deferred Deleted Device Count: 0
 Data loop file: /var/lib/docker/devicemapper/devicemapper/data
 Metadata loop file: /var/lib/docker/devicemapper/devicemapper/metadata
 Library Version: 1.02.107-RHEL7 (2015-12-01)
Execution Driver: native-0.2
Logging Driver: json-file
Kernel Version: 3.10.0-327.el7.x86_64
Operating System: Employee SKU
CPUs: 1
Total Memory: 993 MiB
Name: dhcp.example.com
ID: QPOX:46K6:RZK5:GPBT:DEUD:QM6H:5LRE:R63D:42DI:4BH3:6ZOZ:5EUM
"""

docker_info2 = """
Containers: 0
Images: 0
Server Version: 1.9.1
Storage Driver: devicemapper
 Pool Name: rhel-docker--pool
 Pool Blocksize: 524.3 kB
 Base Device Size: 107.4 GB
 Backing Filesystem: xfs
 Data file:
 Metadata file:
 Data Space Used: 62.39 MB
 Data Space Total: 3.876 GB
 Data Space Available: 3.813 GB
 Metadata Space Used: 40.96 kB
 Metadata Space Total: 8.389 MB
 Metadata Space Available: 8.348 MB
 Udev Sync Supported: true
 Deferred Removal Enabled: true
 Deferred Deletion Enabled: true
 Deferred Deleted Device Count: 0
 Library Version: 1.02.107-RHEL7 (2015-12-01)
Execution Driver: native-0.2
Logging Driver: json-file
Kernel Version: 3.10.0-327.el7.x86_64
Operating System: Employee SKU
CPUs: 1
Total Memory: 993 MiB
Name: dhcp.example.com
ID: QPOX:46K6:RZK5:GPBT:DEUD:QM6H:5LRE:R63D:42DI:4BH3:6ZOZ:5EUM
"""

docker_info3 = """
Cannot connect to the Docker daemon. Is the docker daemon running on this host?
"""


def test_docker_info():
    result = dockerinfo.DockerInfo(context_wrap(docker_info1)).data
    sub_key = ["Data loop file", "Server Version", "Data file"]
    sub_result = dict([(key, result[key]) for key in sub_key])
    expected = {'Data loop file': '/var/lib/docker/devicemapper/devicemapper/data', 'Data file': '/dev/loop0', 'Server Version': '1.9.1'}
    assert expected == sub_result

    result = dockerinfo.DockerInfo(context_wrap(docker_info2)).data
    assert result.get("Data loop file") is None

    result = dockerinfo.DockerInfo(context_wrap(docker_info3)).data
    assert result == {}



import doctest
import pytest

from insights.parsers import mokutil_sbstate
from insights.parsers.mokutil_sbstate import MokutilSbstate
from insights.tests import context_wrap

SECUREBOOT_ENABLED = """
SecureBoot enabled
""".strip()

SECUREBOOT_DISABLED = """
SecureBoot disabled
""".strip()

NOT_SUPPORTED = """
EFI variables are not supported on this system
""".strip()

TEST_CASES = [
    (SECUREBOOT_ENABLED, True),
    (SECUREBOOT_DISABLED, False),
    (NOT_SUPPORTED, None)
]


@pytest.mark.parametrize("output, boolean", TEST_CASES)
def test_mokutil(output, boolean):
    test = MokutilSbstate(context_wrap(output))
    assert test.secureboot_enabled == boolean


def test_doc_examples():
    env = {
        "mokutil": MokutilSbstate(context_wrap(SECUREBOOT_ENABLED)),
    }
    failed, total = doctest.testmod(mokutil_sbstate, globs=env)
    assert failed == 0



"""
EngineLog - file ``var/log/ovirt-engine/engine.log``
====================================================

This is a standard log parser based on the LogFileOutput class.

Sample input::

    2016-05-18 13:21:21,115 INFO [org.ovirt.engine.core.bll.scheduling.policyunits.EvenGuestDistributionBalancePolicyUnit] (DefaultQuartzScheduler_Worker-95) [5bc194fa] There is no host with more than 8 running guests, no balancing is needed
    2016-05-18 14:00:51,272 INFO [org.ovirt.engine.core.vdsbroker.VdsUpdateRunTimeInfo] (DefaultQuartzScheduler_Worker-95) [5bc194fa] VM ADLG8201 ab289661-bbaa-4d27-a67a-ad20626f60f0 moved from PoweringUp --> Paused
    2016-05-18 14:00:51,318 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.

Examples:

    >>> logs = shared[EngineLog]
    >>> 'has paused due to storage I/O problem' in logs
    True
    >>> logs.get('INFO')
    ['2016-05-18 13:21:21,115 INFO [org.ovirt.engine.core.bll.scheduling.policyunits.EvenGuestDistributionBalancePolicyUnit] (DefaultQuartzScheduler_Worker-95) [5bc194fa] There is no host with more than 8 running guests, no balancing is needed',
     '2016-05-18 14:00:51,272 INFO [org.ovirt.engine.core.vdsbroker.VdsUpdateRunTimeInfo] (DefaultQuartzScheduler_Worker-95) [5bc194fa] VM ADLG8201 ab289661-bbaa-4d27-a67a-ad20626f60f0 moved from PoweringUp --> Paused']
    >>> from datetime import datetime
    >>> list(logs.get_after(datetime(2016, 5, 18, 14, 0, 0)))
    ['2016-05-18 14:00:51,272 INFO [org.ovirt.engine.core.vdsbroker.VdsUpdateRunTimeInfo] (DefaultQuartzScheduler_Worker-95) [5bc194fa] VM ADLG8201 ab289661-bbaa-4d27-a67a-ad20626f60f0 moved from PoweringUp --> Paused',
     '2016-05-18 14:00:51,318 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.']

"""
from insights.util import deprecated
from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.engine_log)
class EngineLog(LogFileOutput):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ovirt_engine_log.EngineLog` instead.

    Provide access to ovirt engine logs using the LogFileOutput parser class.
    """
    def __init__(self, *args, **kwargs):
        deprecated(EngineLog, "Import EngineLog from insights.parsers.ovirt_engine_log instead.", "3.3.0")
        super(EngineLog, self).__init__(*args, **kwargs)



import json

from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources import yum_updates
from mock.mock import MagicMock, patch


# Verify that the yum_updates broker correctly executes.
def test_yum_updates_runs_correctly():
    expected_json = json.dumps({
            "releasever": "8",
            "basearch": "x86_64",
            "update_list": {
                "NetworkManager-1:1.22.8-4.el8.x86_64": {
                    "available_updates": [
                        {
                            "package": "NetworkManager-1:1.22.8-5.el8_2.x86_64",
                            "repository": "rhel-8-for-x86_64-baseos-rpms",
                            "basearch": "x86_64",
                            "releasever": "8",
                            "erratum": "RHSA-2020:3011"
                        }
                    ]
                }
            },
            "metadata_time": "2021-01-01T09:39:45Z"
    })
    # setup dnf mock
    with patch("insights.specs.datasources.yum_updates.UpdatesManager", yum_updates.DnfManager):
        yum_updates.dnf = MagicMock()
        yum_updates.dnf.cli = MagicMock()
        yum_updates.hawkey = MagicMock()
        yum_updates.dnf.VERSION = "4.7.0"
        yum_updates.dnf.rpm.detect_releasever.return_value = "8"
        yum_updates.dnf.rpm.basearch.return_value = "x86_64"
        yum_updates.dnf.base.Base().conf.substitutions = {"releasever": "8"}
        repo = MagicMock()
        repo._repo.getTimestamp.return_value = 1609493985
        yum_updates.dnf.base.Base().repos.iter_enabled.return_value = [repo]
        pkg = MagicMock(epoch=1, version="1.22.8", release="4.el8", arch="x86_64")
        pkg.name = "NetworkManager"
        yum_updates.dnf.base.Base().sack.query().installed().run.return_value = [pkg]
        update = MagicMock(epoch=1, version="1.22.8", release="5.el8_2", arch="x86_64",
                    reponame="rhel-8-for-x86_64-baseos-rpms", basearch="x86_64", releasever="8")
        update.name = "NetworkManager"
        update.get_advisories.return_value = [MagicMock(id="RHSA-2020:3011")]
        yum_updates.dnf.base.Base().sack.query().filter.return_value = [update]

        result = yum_updates.yum_updates({})
        assert result is not None
        assert isinstance(result, DatasourceProvider)
        assert expected_json == ''.join(result.content)



from insights.core.spec_factory import glob_file, simple_file, head, first_file
from functools import partial
from insights.core.context import HostArchiveContext
from insights.specs import Specs

simple_file = partial(simple_file, context=HostArchiveContext)
glob_file = partial(glob_file, context=HostArchiveContext)
first_file = partial(first_file, context=HostArchiveContext)


class InsightsArchiveSpecs(Specs):

    abrt_status_bare = simple_file("insights_commands/abrt_status_--bare_True")
    all_installed_rpms = glob_file("insights_commands/rpm_-qa*")
    alternatives_display_python = simple_file("insights_commands/alternatives_--display_python")
    auditctl_rules = simple_file("insights_commands/auditctl_-l")
    auditctl_status = simple_file("insights_commands/auditctl_-s")
    aws_instance_id_doc = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_aws_instance_id_doc")
    aws_instance_id_pkcs7 = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_aws_instance_id_pkcs7")
    awx_manage_check_license = simple_file("insights_commands/awx-manage_check_license")
    awx_manage_print_settings = simple_file("insights_commands/awx-manage_print_settings_INSIGHTS_TRACKING_STATE_SYSTEM_UUID_INSTALL_UUID_TOWER_URL_BASE_AWX_CLEANUP_PATHS_AWX_PROOT_BASE_PATH_LOG_AGGREGATOR_ENABLED_LOG_AGGREGATOR_LEVEL_--format_json")
    azure_instance_id = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_azure_instance_id")
    azure_instance_type = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_azure_instance_type")
    azure_instance_plan = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_azure_instance_plan")
    bios_uuid = simple_file("insights_commands/dmidecode_-s_system-uuid")
    blkid = simple_file("insights_commands/blkid_-c_.dev.null")
    brctl_show = simple_file("insights_commands/brctl_show")
    ceph_df_detail = first_file(["insights_commands/ceph_df_detail_-f_json-pretty", "insights_commands/ceph_df_detail_-f_json"])
    ceph_health_detail = first_file(["insights_commands/ceph_health_detail_-f_json-pretty", "insights_commands/ceph_health_detail_-f_json"])
    ceph_insights = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_ceph_insights")
    ceph_osd_dump = first_file(["insights_commands/ceph_osd_dump_-f_json-pretty", "insights_commands/ceph_osd_dump_-f_json"])
    ceph_osd_tree = first_file(["insights_commands/ceph_osd_tree_-f_json-pretty", "insights_commands/ceph_osd_tree_-f_json"])
    ceph_s = first_file(["insights_commands/ceph_-s_-f_json-pretty", "insights_commands/ceph_-s_-f_json"])
    ceph_v = simple_file("insights_commands/ceph_-v")
    certificates_enddate = first_file([
        "insights_commands/find_.etc.origin.node_.etc.origin.master_.etc.pki_.etc.ipa_.etc.tower.tower.cert_-type_f_-exec_.usr.bin.openssl_x509_-noout_-enddate_-in_-exec_echo_FileName",
        "insights_commands/find_.etc.origin.node_.etc.origin.master_.etc.pki_.etc.ipa_-type_f_-exec_.usr.bin.openssl_x509_-noout_-enddate_-in_-exec_echo_FileName",
        "insights_commands/find_.etc.origin.node_.etc.origin.master_.etc.pki_-type_f_-exec_.usr.bin.openssl_x509_-noout_-enddate_-in_-exec_echo_FileName"
    ])
    chkconfig = simple_file("insights_commands/chkconfig_--list")
    chronyc_sources = simple_file("insights_commands/chronyc_sources")
    corosync_cmapctl = glob_file("insights_commands/corosync-cmapctl*")
    cpupower_frequency_info = simple_file("insights_commands/cpupower_-c_all_frequency-info")
    date = simple_file("insights_commands/date")
    date_utc = simple_file("insights_commands/date_--utc")
    db2ls_a_c = simple_file("insights_commands/usr.local.bin.db2ls_-a_-c")
    df__al = first_file(["insights_commands/df_-al_-x_autofs", "insights_commands/df_-al"])
    df__alP = first_file(["insights_commands/df_-alP_-x_autofs", "insights_commands/df_-alP"])
    df__li = first_file(["insights_commands/df_-li_-x_autofs", "insights_commands/df_-li"])
    dig_dnssec = simple_file("insights_commands/dig_dnssec_._SOA")
    dig_edns = simple_file("insights_commands/dig_edns_0_._SOA")
    dig_noedns = simple_file("insights_commands/dig_noedns_._SOA")
    display_name = simple_file("display_name")
    dnf_module_list = simple_file("insights_commands/dnf_-C_--noplugins_module_list")
    dmesg = simple_file("insights_commands/dmesg")
    dmidecode = simple_file("insights_commands/dmidecode")
    dmsetup_info = simple_file("insights_commands/dmsetup_info_-C")
    dmsetup_status = simple_file("insights_commands/dmsetup_status")
    docker_info = simple_file("insights_commands/docker_info")
    docker_list_containers = simple_file("insights_commands/docker_ps_--all_--no-trunc")
    docker_list_images = simple_file("insights_commands/docker_images_--all_--no-trunc_--digests")
    dotnet_version = simple_file("insights_commands/dotnet_--version")
    doveconf = simple_file("insights_commands/doveconf")
    du_dirs = glob_file("insights_commands/du_-s_-k_*")
    engine_db_query_vdsm_version = simple_file("insights_commands/engine-db-query_--statement_SELECT_vs.vds_name_rpm_version_FROM_vds_dynamic_vd_vds_static_vs_WHERE_vd.vds_id_vs.vds_id_--json")
    ethtool = glob_file("insights_commands/ethtool_*", ignore="ethtool_-.*")
    ethtool_S = glob_file("insights_commands/ethtool_-S_*")
    ethtool_T = glob_file("insights_commands/ethtool_-T_*")
    ethtool_c = glob_file("insights_commands/ethtool_-c_*")
    ethtool_g = glob_file("insights_commands/ethtool_-g_*")
    ethtool_i = glob_file("insights_commands/ethtool_-i_*")
    ethtool_k = glob_file("insights_commands/ethtool_-k_*")
    facter = simple_file("insights_commands/facter")
    fc_match = simple_file("insights_commands/fc-match_-sv_sans_regular_roman_family_fontformat")
    fcoeadm_i = simple_file("insights_commands/fcoeadm_-i")
    findmnt_lo_propagation = simple_file("insights_commands/findmnt_-lo_PROPAGATION")
    firewall_cmd_list_all_zones = simple_file("insights_commands/firewall-cmd_--list-all-zones")
    fw_devices = simple_file("insights_commands/fwupdagent_get-devices")
    fw_security = simple_file("insights_commands/fwupdagent_security_--force")
    gcp_instance_type = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_gcp_instance_type")
    gcp_license_codes = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_gcp_license_codes")
    getcert_list = simple_file("insights_commands/getcert_list")
    getconf_page_size = simple_file("insights_commands/getconf_PAGE_SIZE")
    getenforce = simple_file("insights_commands/getenforce")
    getsebool = simple_file("insights_commands/getsebool_-a")
    grubenv = first_file(["insights_commands/grub2-editenv_list", "/boot/grub2/grubenv", "/boot/efi/EFI/redhat/grubenv"])
    grub1_config_perms = first_file(["insights_commands/ls_-lH_.boot.grub.grub.conf", "insights_commands/ls_-l_.boot.grub.grub.conf"])
    grub_config_perms = first_file(["insights_commands/ls_-lH_.boot.grub2.grub.cfg", "insights_commands/ls_-l_.boot.grub2.grub.cfg"])
    grubby_default_index = simple_file("insights_commands/grubby_--default-index")
    grubby_default_kernel = simple_file("insights_commands/grubby_--default-kernel")
    gluster_v_info = simple_file("insights_commands/gluster_volume_info")
    hammer_task_list = simple_file("insights_commands/hammer_--config_.root..hammer.cli.modules.d.foreman.yml_--output_csv_task_list_--search_state_running_AND_label_Actions_Candlepin_ListenOnCandlepinEvents_OR_label_Actions_Katello_EventQueue_Monitor")
    installed_rpms = head(all_installed_rpms)
    hostname = simple_file("insights_commands/hostname_-f")
    hostname_default = simple_file("insights_commands/hostname")
    hostname_short = simple_file("insights_commands/hostname_-s")
    httpd_M = glob_file("insights_commands/*httpd*_-M")
    httpd_on_nfs = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_httpd_on_nfs")
    httpd_V = glob_file("insights_commands/*httpd*_-V")
    initctl_lst = simple_file("insights_commands/initctl_--system_list")
    ip6tables = simple_file("insights_commands/ip6tables-save")
    ip_addr = simple_file("insights_commands/ip_addr")
    ip_addresses = simple_file("insights_commands/hostname_-I")
    ip_route_show_table_all = simple_file("insights_commands/ip_route_show_table_all")
    ip_s_link = first_file(["insights_commands/ip_-s_-d_link", "insights_commands/ip_-s_link"])
    ipcs_m = simple_file("insights_commands/ipcs_-m")
    ipcs_m_p = simple_file("insights_commands/ipcs_-m_-p")
    ipcs_s = simple_file("insights_commands/ipcs_-s")
    iptables = simple_file("insights_commands/iptables-save")
    ipv4_neigh = simple_file("insights_commands/ip_-4_neighbor_show_nud_all")
    ipv6_neigh = simple_file("insights_commands/ip_-6_neighbor_show_nud_all")
    iris_list = simple_file("insights_commands/iris_list")
    iscsiadm_m_session = simple_file("insights_commands/iscsiadm_-m_session")
    journal_header = simple_file("insights_commands/journalctl_--no-pager_--header")
    keystone_crontab = first_file(["insights_commands/crontab_-l_-u_keystone", "var/spool/cron/keystone"])
    kpatch_list = simple_file("insights_commands/kpatch_list")
    localtime = simple_file("insights_commands/file_-L_.etc.localtime")
    losetup = simple_file("insights_commands/losetup_-l")
    lpstat_p = simple_file("insights_commands/lpstat_-p")
    ls_boot = simple_file("insights_commands/ls_-lanR_.boot")
    ls_dev = simple_file("insights_commands/ls_-lanR_.dev")
    ls_disk = simple_file("insights_commands/ls_-lanR_.dev.disk")
    ls_edac_mc = simple_file("insights_commands/ls_-lan_.sys.devices.system.edac.mc")
    ls_etc = first_file(["insights_commands/ls_-lan_.etc_.etc.cloud.cloud.cfg.d_.etc.nova.migration_.etc.pki.ovirt-vmconsole_.etc.pki.tls.certs_.etc.pki.tls.private_.etc.rc.d.init.d_.etc.selinux.targeted.policy_.etc.sysconfig_.etc.watchdog.d", "insights_commands/ls_-lan_.etc_.etc.cloud.cloud.cfg.d_.etc.nova.migration_.etc.pki.ovirt-vmconsole_.etc.pki.tls.certs_.etc.pki.tls.private_.etc.rc.d.init.d_.etc.selinux.targeted.policy_.etc.sysconfig", "insights_commands/ls_-lan_.etc_.etc.cloud.cloud.cfg.d_.etc.nova.migration_.etc.pki.ovirt-vmconsole_.etc.pki.tls.certs_.etc.pki.tls.private_.etc.rc.d.init.d_.etc.sysconfig"])
    ls_etc_ssh = simple_file("insights_commands/ls_-lanL_.etc.ssh")
    ls_ipa_idoverride_memberof = simple_file("insights_commands/ls_-lan_.usr.share.ipa.ui.js.plugins.idoverride-memberof")
    ls_krb5_sssd = simple_file("insights_commands/ls_-lan_.var.lib.sss.pubconf.krb5.include.d")
    ls_lib_firmware = simple_file("insights_commands/ls_-lanR_.lib.firmware")
    ls_ocp_cni_openshift_sdn = simple_file("insights_commands/ls_-l_.var.lib.cni.networks.openshift-sdn")
    ls_origin_local_volumes_pods = simple_file("insights_commands/ls_-l_.var.lib.origin.openshift.local.volumes.pods")
    ls_osroot = simple_file("insights_commands/ls_-lan")
    ls_R_var_lib_nova_instances = simple_file("insights_commands/ls_-laR_.var.lib.nova.instances")
    ls_sys_firmware = simple_file("insights_commands/ls_-lanR_.sys.firmware")
    ls_systemd_units = simple_file("insights_commands/ls_-lanRL_.etc.systemd_.run.systemd_.usr.lib.systemd_.usr.local.lib.systemd")
    ls_tmp = simple_file("insights_commands/ls_-la_.tmp")
    ls_usr_bin = simple_file("insights_commands/ls_-lan_.usr.bin")
    ls_usr_lib64 = simple_file("insights_commands/ls_-lan_.usr.lib64")
    ls_var_cache_pulp = simple_file("insights_commands/ls_-lan_.var.cache.pulp")
    ls_var_lib_mongodb = simple_file("insights_commands/ls_-la_.var.lib.mongodb")
    ls_var_lib_nova_instances = simple_file("insights_commands/ls_-laRZ_.var.lib.nova.instances")
    ls_var_lib_pcp = simple_file("insights_commands/ls_-la_.var.lib.pcp")
    ls_var_lib_rpm = simple_file("insights_commands/ls_-lan_.var.lib.rpm")
    ls_var_lib_rsyslog = simple_file("insights_commands/ls_-lZ_.var.lib.rsyslog")
    ls_var_log = simple_file("insights_commands/ls_-la_.var.log_.var.log.audit")
    ls_var_opt_mssql = simple_file("insights_commands/ls_-ld_.var.opt.mssql")
    ls_var_opt_mssql_log = simple_file("insights_commands/ls_-la_.var.opt.mssql.log")
    ls_var_spool_clientmq = simple_file("insights_commands/ls_-ln_.var.spool.clientmqueue")
    ls_var_spool_postfix_maildrop = simple_file("insights_commands/ls_-ln_.var.spool.postfix.maildrop")
    ls_var_tmp = simple_file("insights_commands/ls_-ln_.var.tmp")
    ls_var_run = simple_file("insights_commands/ls_-lnL_.var.run")
    ls_var_www = simple_file("insights_commands/ls_-la_.dev.null_.var.www")
    lsblk = simple_file("insights_commands/lsblk")
    lsblk_pairs = simple_file("insights_commands/lsblk_-P_-o_NAME_KNAME_MAJ_MIN_FSTYPE_MOUNTPOINT_LABEL_UUID_RA_RO_RM_MODEL_SIZE_STATE_OWNER_GROUP_MODE_ALIGNMENT_MIN-IO_OPT-IO_PHY-SEC_LOG-SEC_ROTA_SCHED_RQ-SIZE_TYPE_DISC-ALN_DISC-GRAN_DISC-MAX_DISC-ZERO")
    lscpu = simple_file("insights_commands/lscpu")
    lsmod = simple_file("insights_commands/lsmod")
    lsof = simple_file("insights_commands/lsof")
    lspci = simple_file("insights_commands/lspci_-k")
    lspci_vmmkn = simple_file("insights_commands/lspci_-vmmkn")
    lssap = simple_file("insights_commands/usr.sap.hostctrl.exe.lssap")
    lsscsi = simple_file("insights_commands/lsscsi")
    lvmconfig = first_file([
        "insights_commands/lvmconfig_--type_full",
        "insights_commands/lvm_dumpconfig_--type_full"
    ])
    lvs_noheadings = first_file(
        [
            "insights_commands/lvs_--nameprefixes_--noheadings_--separator_-a_-o_lv_name_lv_size_lv_attr_mirror_log_vg_name_devices_region_size_data_percent_metadata_percent_segtype_seg_monitor_lv_kernel_major_lv_kernel_minor_--config_global_locking_type_0",
            "insights_commands/lvs_--nameprefixes_--noheadings_--separator_-a_-o_lv_name_lv_size_lv_attr_mirror_log_vg_name_devices_region_size_data_percent_metadata_percent_segtype_seg_monitor_--config_global_locking_type_0"
        ]
    )
    max_uid = simple_file("insights_commands/awk_-F_if_3_max_max_3_END_print_max_.etc.passwd")
    md5chk_files = glob_file("insights_commands/md5sum_*")
    mount = simple_file("insights_commands/mount")
    modinfo = glob_file("insights_commands/modinfo_*")
    mokutil_sbstate = simple_file("insights_commands/mokutil_--sb-state")
    mpirun_version = simple_file("insights_commands/usr.local.bin.mpirun_--version")
    multicast_querier = simple_file("insights_commands/find_.sys.devices.virtual.net._-name_multicast_querier_-print_-exec_cat")
    multipath_conf_initramfs = simple_file("insights_commands/lsinitrd_-f_.etc.multipath.conf")
    multipath__v4__ll = simple_file("insights_commands/multipath_-v4_-ll")
    mysqladmin_vars = simple_file("insights_commands/mysqladmin_variables")
    named_checkconf_p = simple_file("insights_commands/named-checkconf_-p")
    ndctl_list_Ni = simple_file("insights_commands/ndctl_list_-Ni")
    netstat = simple_file("insights_commands/netstat_-neopa")
    netstat_agn = simple_file("insights_commands/netstat_-agn")
    netstat_i = simple_file("insights_commands/netstat_-i")
    netstat_s = simple_file("insights_commands/netstat_-s")
    nmcli_conn_show = simple_file("insights_commands/nmcli_conn_show")
    nmcli_dev_show = simple_file("insights_commands/nmcli_dev_show")
    nova_crontab = first_file(["insights_commands/crontab_-l_-u_nova", "var/spool/cron/nova"])
    nova_uid = simple_file("insights_commands/id_-u_nova")
    ntpq_leap = simple_file("insights_commands/ntpq_-c_rv_0_leap")
    ntpq_pn = simple_file("insights_commands/ntpq_-pn")
    ntptime = simple_file("insights_commands/ntptime")
    numeric_user_group_name = simple_file("insights_commands/grep_-c_digit_.etc.passwd_.etc.group")
    oc_get_clusterrole_with_config = simple_file("insights_commands/oc_get_clusterrole_--config_.etc.origin.master.admin.kubeconfig")
    oc_get_clusterrolebinding_with_config = simple_file("insights_commands/oc_get_clusterrolebinding_--config_.etc.origin.master.admin.kubeconfig")
    od_cpu_dma_latency = simple_file("insights_commands/od_-An_-t_d_.dev.cpu_dma_latency")
    open_vm_tools_stat_raw_text_session = simple_file("insights_commands/vmware-toolbox-cmd_stat_raw_text_session")
    openvswitch_other_config = simple_file("insights_commands/ovs-vsctl_-t_5_get_Open_vSwitch_._other_config")
    ovs_vsctl_list_bridge = simple_file("insights_commands/ovs-vsctl_list_bridge")
    ovs_vsctl_show = simple_file("insights_commands/ovs-vsctl_show")
    package_provides_command = glob_file("insights_commands/echo_*java*")
    parted__l = simple_file("insights_commands/parted_-l_-s")
    passenger_status = simple_file("insights_commands/passenger-status")
    pci_rport_target_disk_paths = simple_file("insights_commands/find_.sys.devices._-maxdepth_10_-mindepth_9_-name_stat_-type_f")
    pcp_metrics = simple_file("insights_commands/curl_-s_http_..127.0.0.1_44322.metrics_--connect-timeout_5")
    pcs_quorum_status = simple_file("insights_commands/pcs_quorum_status")
    pcs_status = simple_file("insights_commands/pcs_status")
    pmrep_metrics = first_file(["insights_commands/pmrep_-t_1s_-T_1s_network.interface.out.packets_network.interface.collisions_swap.pagesout_mssql.memory_manager.stolen_server_memory_mssql.memory_manager.total_server_memory_-o_csv", "insights_commands/pmrep_-t_1s_-T_1s_network.interface.out.packets_network.interface.collisions_swap.pagesout_-o_csv"])
    podman_list_containers = simple_file("insights_commands/podman_ps_--all_--no-trunc")
    podman_list_images = simple_file("insights_commands/podman_images_--all_--no-trunc_--digests")
    postconf_builtin = simple_file("insights_commands/postconf_-C_builtin")
    postconf = simple_file("insights_commands/postconf")
    ps_alxwww = simple_file("insights_commands/ps_alxwww")
    ps_aux = simple_file("insights_commands/ps_aux")
    ps_auxcww = simple_file("insights_commands/ps_auxcww")
    ps_auxww = simple_file("insights_commands/ps_auxww")
    ps_ef = simple_file("insights_commands/ps_-ef")
    ps_eo = simple_file("insights_commands/ps_-eo_pid_ppid_comm")
    puppet_ca_cert_expire_date = simple_file("insights_commands/openssl_x509_-in_.etc.puppetlabs.puppet.ssl.ca.ca_crt.pem_-enddate_-noout")
    pvs_noheadings = simple_file("insights_commands/pvs_--nameprefixes_--noheadings_--separator_-a_-o_pv_all_vg_name_--config_global_locking_type_0")
    rabbitmq_report = simple_file("insights_commands/rabbitmqctl_report")
    rabbitmq_users = simple_file("insights_commands/rabbitmqctl_list_users")
    readlink_e_etc_mtab = simple_file("insights_commands/readlink_-e_.etc.mtab")
    readlink_e_shift_cert_client = simple_file("insights_commands/readlink_-e_.etc.origin.node.certificates.kubelet-client-current.pem")
    readlink_e_shift_cert_server = simple_file("insights_commands/readlink_-e_.etc.origin.node.certificates.kubelet-server-current.pem")
    repquota_agnpuv = simple_file("insights_commands/repquota_-agnpuv")
    rhev_data_center = simple_file("insights_commands/python_-m_insights.tools.cat_--no-header_rhev_data_center")
    rhsm_katello_default_ca_cert = simple_file("insights_commands/openssl_x509_-in_.etc.rhsm.ca.katello-default-ca.pem_-noout_-issuer")
    rndc_status = simple_file("insights_commands/rndc_status")
    rpm_ostree_status = simple_file("insights_commands/rpm-ostree_status_--json")
    rpm_V_packages = first_file(["insights_commands/rpm_-V_coreutils_procps_procps-ng_shadow-utils_passwd_sudo_chrony_findutils", "insights_commands/rpm_-V_coreutils_procps_procps-ng_shadow-utils_passwd_sudo_chrony", "insights_commands/rpm_-V_coreutils_procps_procps-ng_shadow-utils_passwd_sudo"])
    saphostctl_getcimobject_sapinstance = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostctrl_-function_GetCIMObject_-enuminstances_SAPInstance")
    saphostexec_status = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostexec_-status")
    saphostexec_version = simple_file("insights_commands/usr.sap.hostctrl.exe.saphostexec_-version")
    satellite_content_hosts_count = first_file([
        "insights_commands/sudo_-iu_postgres_.usr.bin.psql_-d_foreman_-c_select_count_from_hosts",
        "insights_commands/sudo_-iu_postgres_psql_-d_foreman_-c_select_count_from_hosts"
    ])
    satellite_custom_ca_chain = simple_file("insights_commands/awk_BEGIN_pipe_openssl_x509_-noout_-subject_-enddate_._-_BEGIN_CERT._._-_END_CERT._print_pipe_._-_END_CERT._close_pipe_printf_n_.etc.pki.katello.certs.katello-server-ca.crt")
    satellite_provision_param_settings = simple_file('insights_commands/sudo_-iu_postgres_.usr.bin.psql_-d_foreman_-c_select_name_value_from_parameters_where_name_package_upgrade_and_reference_id_in_select_id_from_operatingsystems_where_name_RedHat_and_major_9_--csv')
    satellite_qualified_capsules = simple_file("insights_commands/sudo_-iu_postgres_.usr.bin.psql_-d_foreman_-c_select_name_from_smart_proxies_where_download_policy_background_--csv")
    satellite_qualified_katello_repos = simple_file("insights_commands/sudo_-iu_postgres_.usr.bin.psql_-d_foreman_-c_select_id_name_url_download_policy_from_katello_root_repositories_where_download_policy_background_or_url_is_NULL_--csv")
    sealert = simple_file('insights_commands/sealert_-l')
    sestatus = simple_file("insights_commands/sestatus_-b")
    smbstatus_p = simple_file("insights_commands/smbstatus_-p")
    software_collections_list = simple_file('insights_commands/scl_--list')
    spamassassin_channels = simple_file('insights_commands/grep_-r_s_CHANNELURL_.etc.mail.spamassassin.channel.d')
    ss = simple_file("insights_commands/ss_-tupna")
    sshd_config_perms = first_file(["insights_commands/ls_-lH_.etc.ssh.sshd_config", "insights_commands/ls_-l_.etc.ssh.sshd_config"])
    subscription_manager_facts = simple_file("insights_commands/subscription-manager_facts")
    subscription_manager_id = simple_file("insights_commands/subscription-manager_identity")
    subscription_manager_installed_product_ids = simple_file("insights_commands/find_.etc.pki.product-default._.etc.pki.product._-name_pem_-exec_rct_cat-cert_--no-content")
    sysctl = simple_file("insights_commands/sysctl_-a")
    systemctl_cat_dnsmasq_service = simple_file("insights_commands/systemctl_cat_dnsmasq.service")
    systemctl_cat_rpcbind_socket = simple_file("insights_commands/systemctl_cat_rpcbind.socket")
    systemctl_list_unit_files = simple_file("insights_commands/systemctl_list-unit-files")
    systemctl_list_units = simple_file("insights_commands/systemctl_list-units")
    systemctl_show_all_services = simple_file("insights_commands/systemctl_show_.service")
    systemctl_show_all_services_with_limited_properties = simple_file("insights_commands/systemctl_show_.service_--all_--property_Names_CPUAccounting_CPUQuotaPerSecUSec_CPUShares_StartupCPUShares_UnitFileState_SubState")
    systemctl_show_target = simple_file("insights_commands/systemctl_show_.target")
    systemctl_smartpdc = simple_file("insights_commands/systemctl_show_smart_proxy_dynflow_core")
    systemd_analyze_blame = simple_file("insights_commands/systemd-analyze_blame")
    systemd_docker = first_file(["insights_commands/systemctl_cat_docker.service", "/usr/lib/systemd/system/docker.service"])
    systemd_openshift_node = first_file(["insights_commands/systemctl_cat_atomic-openshift-node.service", "/usr/lib/systemd/system/atomic-openshift-node.service"])
    systool_b_scsi_v = simple_file("insights_commands/systool_-b_scsi_-v")
    testparm_s = simple_file("insights_commands/testparm_-s")
    testparm_v_s = simple_file("insights_commands/testparm_-v_-s")
    timedatectl_status = simple_file("insights_commands/timedatectl_status")
    tomcat_vdc_fallback = simple_file("insights_commands/find_.usr.share_-maxdepth_1_-name_tomcat_-exec_.bin.grep_-R_-s_VirtualDirContext_--include_.xml")
    tuned_adm = simple_file("insights_commands/tuned-adm_list")
    uname = simple_file("insights_commands/uname_-a")
    uptime = simple_file("insights_commands/uptime")
    vdo_status = simple_file("insights_commands/vdo_status")
    vgdisplay = simple_file("insights_commands/vgdisplay")
    vgs_noheadings = simple_file("insights_commands/vgs_--nameprefixes_--noheadings_--separator_-a_-o_vg_all_--config_global_locking_type_0")
    virsh_list_all = simple_file("insights_commands/virsh_--readonly_list_--all")
    virt_what = simple_file("insights_commands/virt-what")
    wc_proc_1_mountinfo = simple_file("insights_commands/wc_-l_.proc.1.mountinfo")
    xfs_quota_state = simple_file("insights_commands/xfs_quota_-x_-c_state_-gu")
    yum_list_available = simple_file("insights_commands/yum_-C_--noplugins_list_available")
    yum_repolist = first_file([
        "insights_commands/yum_-d_2_-C_--noplugins_repolist",
        "insights_commands/yum_-C_--noplugins_repolist",
        "insights_commands/yum_-C_repolist"])
    yum_updateinfo = simple_file("insights_commands/yum_-C_updateinfo_list")
